#!/bin/ksh
#
#  ----------------------------------------------------------------------------
#  (C) COPYRIGHT International Business Machines Corp. 2001
#  All Rights Reserved
#  Licensed Materials - Property of IBM
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#@ ----------------------------------------------------------------------------
#@
#@ Script Name: h27946a3.ec
#@
#@
#@ Component: MES EC Upgrade
#@
#@
#@ Description:
#@
#@   This script upgrades C00 and C01 systems currently running AIX 4.3.3.x
#@   and Tivoli Storage Manager (TSM) 4.1.x.x to AIX 5.1.0.x and TSM 4.2.1.x,
#@   via an operating system replacement image on a 4mm mksysb backup tape.
#@   Critical files and configuration will be saved onto diskette for
#@   restoration and verification when the new image is loaded.
#@
#@   The following scripts are used by this EC:
#@
#@   vdisk
#@   verify3466ssa
#@   verify3466tsm
#@   captureIPenvironment
#@   restoreIPenvironment
#@   getTapeInfo
#@   tapeRec
#@
#@
#@ Input:
#@
#@   Positional parameters: None
#@
#@   StorageSmart Build Environment Variables
#@
#@
#@ Special options:
#@
#@   -n   Do not utilize the revfile log file logging routine when this script
#@        is called by another script.
#@
#@   -h   Display this help text.
#@
#@
#@ Output:
#@
#@   Messages written to STDOUT
#@
#@   Return Code  0: Successful
#@                1: Failed
#@
#@
#@ ----------------------------------------------------------------------------
#@
#@ Example Usage:
#@
#@   h27946a3.ec
#@   h27946a3.ec -n
#@   h27946a3.ec -h
#@
#@ ----------------------------------------------------------------------------
#
# Change History:
#
# Date         Name                 Description
# ----------   ------------------   -------------------------------------------
# 08.08.2001   Howard Jiang         Create
# 04.19.2002   Howard Jiang         Create symbolic link for core file in
#                                   Phase 2
# 04.29.2002   Howard Jiang         Pass on TSM storage pool migration info if
#                                   Phase 1 is executed multiple times
# 06.24.2002   Howard Jiang         Fixed 24hr dbbackup check
#
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Input validation
# -----------------------------------------------------------------------------

scriptName='h27946a3.ec'
hostName=`hostname`
programRC=0
whichPhase=''
noLoggingOption='no'
adsmlogDir="/adsmlog"
prepareDir="$adsmlogDir/prepare"
prepareDirTsmServer="$prepareDir/tsmServer"
prepareDirTsmClient="$prepareDir/tsmClient"
prepareDirSysInfo="$prepareDir/sysInfo"
minLogDirFreeSpace='250000'
primaryServerName='primary'
approvedTsmVersion='4'
approvedTsmRelease='1'
completedTsmVersion='4'
completedTsmRelease='2'
subprocessTypeFunction='function'
subprocessTypeScript='script'
subprocessTypeEntering='Entering'
subprocessTypeExiting='Exiting'
subprocessTypeSkipping='Skipping'
export thisEcLevel='H27946A'
export thisEcLevel2='H27946'
maxSizeOfDiskette="1464516"

# -------------
# Files to save
# -------------

nsmMainDir="/usr/nsm.3466"

pathToTsmClientFiles="/usr/tivoli/tsm/client/ba/bin"
tsmClientFiles="dsm.opt dsm.sys"
pathToTsmServerFiles="/usr/tivoli/tsm/server/bin"
tsmServerFiles="devconfg.file dsmserv.dsk dsmserv.opt volhist.file"
pathToEtcFiles="/etc"
etcFiles="hosts ibmatl.conf resolv.conf"
etcFiles2="group noOptionsForADSM.stsm passwd"
pathToRootFiles="/"
rootFiles="profile.bak"
pathToSysInfoFiles="$nsmMainDir/sysInfo"
sysInfoFiles="3466.model sysInfoaa1.out sysInfoaa2.out"
relativePathToSsaDiskFiles="ssa_disks"
pathToSsaDiskFiles="$nsmMainDir/$relativePathToSsaDiskFiles"
ssaDiskFiles=`ls $nsmMainDir/$relativePathToSsaDiskFiles`
relativePathToSsLevelFiles="webshell"
pathToSsLevelFiles="/var/adm"
ssLevelFiles=`ls /var/adm | grep WebShlLvl`
relativePathToDrm1Files="drm1"
relativePathToDrm2Files="drm2"
nodesDatFile="nodes.dat"
pathToNodesDatFile="/usr/lpp/internet/server_root/admin-bin"

genericSysInfo="genericSysInfo"
criticalSysInfo="criticalSysInfo"
storagePoolMigration="stgpoolmig.lst"
getTapeInfoBeforeFile="getTapeInfo.before"
getTapeInfoAfterFile="getTapeInfo.after"
tapeRecMarkerFile="tapeReconciliationMarker"
captureIPenvironmentFile="systemIPinfo"
listOfFailedFiles=""

archivedFiles="phase1.tar"
restoreDir="/tmp/$scriptName.phase1"
targetOsLevel="5.1.0.0"
extraFiles="/core /3466Env /3466skulker /etc/exclude.rootvg $pathToTsmServerFiles/core"
extraDirs="/dummyDirToCheckBosinstInstall"

# ---------------------
# Development Overrides
# ---------------------

# Pre-analysis Phase

skipExpandScreen='no'
skipCheckForExistingEc='no'
skipDetermineProcessorInfo='no'
skipDisplayKinServerWarning='no'
skipVerifyNsmMainDir='no'
skipDetermineModelNumber='no'
skipVerifyAdsmlogDir='no'
skipStartPrimaryServer='no'
skipVerifyServerCommunications='no'
skipVerifyServerCommAutomatically='no'
skipAssignTsmAdminVariables='no'
skipVerifyApprovedTsmLevel='no'
skipCallVerify3466Scripts='no'

# -------
# Phase 1

skipGatherSysInfo='no'
skipCheckForFullDatabaseBackup='no'
skipCollectTsmLicenseInformation='no'
skipDisableMigration='no'
skipDisableReclamation='no'
skipDisableSchedules='no'
skipDisableSessions='no'
skipCollectTapeInformation='no'
skipCollectNetworkInformation='no'
skipCheckMiscTsmServerFileSizes='no'
skipFormatFloppy='no'
skipSaveDrmFiles='no'
skipSaveFilesToFloppy='no'
skipStopPrimaryServer='no'
skipPrebackupExportit='no'
skipEndPhase1='no'

# -------
# Phase 2

skipCallFixSsa='no'
skipRestoreFilesFromFloppy='no'
skipUpdateModel='no'
skipCheckSystemDateTz='no'
skipPostbackupImportit='no'
skipCheckSsa='no'
skipRemoveExtraFiles='no'
skipRemoveRmt0='no'
skipRestoreDrmFiles='no'			# NEVER skip
skipRestoreFilesToOriginalLocations='no'
skipUpgradeDb='no'
skipRestoreNetworkInformation='no'
skipRegisterTsmLicenses='no'
skipPerformTapeReconciliation='no'
skipEnableSchedules='no'
skipAuditTsmLicenses='no'
skipAllowTsmProcessesToFinish='no'
skipEnableMigration='no'
skipEnableReclamation='no'
skipApplyEcStamp='no'
skipCallRecordSsa='no'
skipAddNodesDat='no'
skipEndPhase2='no'

# -----------------------------------------------------------------------------
# Display error messages to STDERR
# -----------------------------------------------------------------------------

writeErr ()
{
  print "$*" >&2
}

# -----------------------------------------------------------------------------
# Displays a message when entering, exiting, or skipping a subprocess
# (either a function or script)
#
# Usage: subprocessStatus $subprocessName (defined in the function)
#                         $subprocessTypeFunction/Script (pre-defined vars)
#                         $subprocessTypeEntering/Exiting/Skipping (pre-def)
# -----------------------------------------------------------------------------

function subprocessStatus {

  currentTime=`date "+%I:%M:%S %p"`
  currentDate=`date "+%m.%d.%Y"`

  if [ "$3" = 'Entering' ]
  then
    symbolIdentifier='>'
  elif [ "$3" = 'Exiting' ]
  then
    symbolIdentifier='<'
  elif [ "$3" = 'Skipping' ]
  then
    symbolIdentifier='+'
  fi

  print "INFO: ($scriptName) $symbolIdentifier $3 $2: $1{}"
  print "                      [$currentTime] -*- [$currentDate]"
  print

}  # END subprocessStatus

# -----------------------------------------------------------------------------
# Prompts the user to type 'c' and Enter to continue
# -----------------------------------------------------------------------------

function cAndEnterToContinue {

  response=''
  until [ "$response" = 'c' ] || [ "$response" = 'C' ]
  do
    read response?"Press 'C' and [Enter] to continue: "
    echo
  done

}  # END cAndEnterToContinue

# -----------------------------------------------------------------------------
# Prompts the user to type 'e' and Enter to exit
# -----------------------------------------------------------------------------

function eAndEnterToExit {

  response=''
  until [ "$response" = 'e' ] || [ "$response" = 'E' ]
  do
    read response?"Press 'E' and [Enter] to exit: "
    echo
  done
  programRC=1
  exit $programRC

}  # END eAndEnterToExit

# -----------------------------------------------------------------------------
# Prompts the user to type 'c' and Enter to continue or 'e' and Enter to exit
# -----------------------------------------------------------------------------

function cOrEAndEnterToContinue {

  response=''
  until [ "$response" = 'c' ] || [ "$response" = 'C' ] || [ "$response" = 'e' ] || [ "$res
ponse" = 'E' ]
  do
    read response?"Press 'C' and [Enter] to continue or 'E' and [Enter] to exit: "
    echo
  done

  if [ "$response" = 'e' ] || [ "$response" = 'E' ]
  then
    programRC=1
    exit $programRC
  fi

}  # END cOrEAndEnterToContinue

# -----------------------------------------------------------------------------
# Sleeps for a certain amount of time and displays a spinner
# -----------------------------------------------------------------------------

function sleepWithSpinner {

  theNumberOfSecondsToSleep="$1"
  sleepCount=0
  print "INFO: ($scriptName) - Sleeping for $theNumberOfSecondsToSleep seconds: -\c"
  while [ $sleepCount -lt $theNumberOfSecondsToSleep ]
  do
    sleep 1
    print "\b\ \b\c"
    sleepCount=$$(($sleepCount+1))
    sleep 1
    echo '\b|\c'
    sleepCount=$$(($sleepCount+1))
    sleep 1
    echo '\b/\c'
    sleepCount=$$(($sleepCount+1))
    sleep 1
    echo '\b-\c'
    sleepCount=$$(($sleepCount+1))
  done
  print
  print

}  # END sleepWithSpinner

# -----------------------------------------------------------------------------
# Tell the user to expand the screen
# -----------------------------------------------------------------------------

function expandScreen {

  subprocessName='expandScreen'
  if [ "$skipExpandScreen" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    echo
    echo "+-----------------------------------------------------------------------------+"
    echo "|                                                                             |"
    echo "| ! ATTENTION !                                                               |"
    echo "|                                                                             |"
    echo "| Information will be displayed on the following screens that will not fit in |"
    echo "| a window which was launched with the default size of 80x24.                 |"
    echo "|                                                                             |"
    echo "| You should increase the size of this window to at least 100x42 in order to  |"
    echo "| effectively view information presented in this script.                      |"
    echo "|                                                                             |"
    echo "| The window size can be increased by:                                        |"
    echo "|                                                                             |"
    echo "|  -- Maximizing to full screen with the maximize button                      |"
    echo "|     (upper right hand corner)                                               |"
    echo "|                                                                             |"
    echo "|  -- Stretching the window height.  Position the mouse on the top or bottom  |"
    echo "|     edge of the screen.  Notice that the pointer changes to indicate the    |"
    echo "|     mouse is positioned to stretch the screen.  Hold down the left mouse    |"
    echo "|     button and move the mouse to stretch the screen.                        |"
    echo "|                                                                             |"
    echo "+-----------------------------------------------------------------------------+"
    echo
    cOrEAndEnterToContinue

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END expandScreen

# ---------------------------------------------------------------------------
# See if this EC has already been applied
# ---------------------------------------------------------------------------

function checkForExistingEc {

  subprocessName='checkForExistingEc'
  if [ "$skipCheckForExistingEc" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    currentSsLevel=`lsSSlevel | grep $thisEcLevel`

    if [ "$currentSsLevel" != '' ]
    then
      currentPhase=2
    else
      currentEcFound=$(lsSSlevel | awk '{print $1}' | grep -v 'The' | grep $thisEcLevel2)
      if [ "$currentEcFound" = '' ]
      then
        print "INFO: ($scriptName) - This EC has not been previously applied."
        print
      else
        writeErr "NSM-95100-E: ERROR($scriptName) - $thisEcLevel2 has already been applied!"
        writeErr
        eAndEnterToExit
      fi
      currentPhase=1
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END checkForExistingEc

# ---------------------------------------------------------------------------
# Identify the RS/6000 processor type and model
# ---------------------------------------------------------------------------

function determineProcessorInfo {

  subprocessName='determineProcessorInfo'
  if [ "$skipDetermineProcessorInfo" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    theMachine=`lsattr -El sys0 | grep modelname | awk '{print $2}' | cut -f2 -d","`
    theMachineType=`print $theMachine | cut -f2 -d"-"`
    theMachineModel=`print $theMachine | cut -f1 -d"-"`

    print "INFO: ($scriptName) - Processor is a $theMachineType Model $theMachineModel."
    print

    if [ "$theMachineType" = "H50" ] || [ "$theMachineType" = "H70" ]
    then
      export processorType="$theMachineType"
      if [ "$theMachineType" = "H50" ]
      then
        export allowH50='yes'  # Used for reportTape/getTapeInfo
      fi
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      writeErr "NSM-95101-E: ERROR($scriptName) - You must have a H50 or H70 to proceed with this upgrade!"
      writeErr
      eAndEnterToExit
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END determineProcessorInfo

# ---------------------------------------------------------------------------
# Display a warning to take of kin servers manually
# ---------------------------------------------------------------------------

function displayKinServerWarning {

  subprocessName='displayKinServerWarning'
  if [ "$skipDisplayKinServerWarning" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    listOfKinServers=`ls / | grep adsmkin`
    if [ "$listOfKinServers" = '' ]
    then
      print "INFO: ($scriptName) - No kin servers found."
      print
    else
      for eachKinServer in $listOfKinServers
      do
        if [[ -f /$eachKinServer/dsmserv.dsk ]]
        then
          export ignoreKinServers='yes'  # Used for verify3466tsm
        fi
      done

      function displayHeader {
        print
        print "+-----------------------------------------------------------------------------+"
        print "|                                                                             |"
        print "|   ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING !   |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "| One or more kin servers were found on this system.  This EC will NOT        |"
        print "| upgrade those servers automatically.  It is the responsibility of the       |"
        print "| person applying this EC to save and migrate the configuration files         |"
        print "| manually.  Failure to do so will result in the loss of those servers.  You  |"
        print "| may exit now to collect the necessary information before restarting this    |"
        print "| script.                                                                     |"
        print "|                                                                             |"
        print "| If any kin servers are found to be running, you will be given a chance to   |"
        print "| stop them later in this EC.  If you do not wish to save any kin servers,    |"
        print "| you may ignore this message and continue.                                   |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|   ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING ! WARNING !   |"
        print "|                                                                             |"
        print "+-----------------------------------------------------------------------------+"
        print
        print
      }  # END displayHeader

      function displayMenu {
        print "[1] Yes, I understand and have collected the necessary information.  Continue."
        print
        print "[2] Yes, I understand and do not wish to save any kin servers.  Continue."
        print
        print "[3] No, I want to exit this script."
        print
      }  # END displayMenu

      function userSelect {
        displayHeader
        displayMenu
        response=0
        until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
        do
          read response?"Please make your selection [1,2,3]: "
          if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
          then
            : # Pass through
          else
            displayHeader
            print "-> $response <- is an invalid response!  Please try again."
            print
            print
            displayMenu
          fi
        done

        case $response in
          (1)
            print
            print "INFO: ($scriptName) - User selected Option [1] -- Continuing script."
            print
            ;;
          (2)
            print
            print "INFO: ($scriptName) - User selected Option [2] -- Continuing script."
            print
            ;;
          (3)
            print
            print "INFO: ($scriptName) - User selected Option [3] -- Exiting script."
            print
            programRC=1
            exit $programRC
            ;;
        esac
      }  # END userSelect

      userSelect
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END displayKinServerWarning

# ---------------------------------------------------------------------------
# Check for the minimum level of the NSM script package
# ---------------------------------------------------------------------------

function verifyNsmPackageLevel {

  subprocessName='verifyNsmPackageLevel'
  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

  minimumV=4  # Minimum version
  minimumR=2  # Minimum release
  minimumM=1  # Minimum maintenance
  minimumF=0  # Minimum fix

  levelOfNsmPackage=`lslpp -l | grep nsm.3466 | awk '{print $2}'`
  currentV=`print $levelOfNsmPackage | cut -f1 -d'.'`
  currentR=`print $levelOfNsmPackage | cut -f2 -d'.'`
  currentM=`print $levelOfNsmPackage | cut -f3 -d'.'`
  currentF=`print $levelOfNsmPackage | cut -f4 -d'.'`

  print "INFO: ($scriptName) - Checking NSM script package level ..."
  print
  print "                      Minimum level: $minimumV.$minimumR.$minimumM.$minimumF"
  print "                      Current level: $currentV.$currentR.$currentM.$currentF"
  print

  if [ "$currentV" -ge "$minimumV" ]
  then
    if [ "$currentV" -gt "$minimumV" ]
    then
      displayError='no'
    else
      if [ "$currentR" -ge "$minimumR" ]
      then
        if [ "$currentR" -gt "$minimumR" ]
        then
          displayError='no'
        else
          if [ "$currentM" -ge "$minimumM" ]
          then
            displayError='no'
          else
            displayError='yes'
          fi
        fi
      else
        displayError='yes'
      fi
    fi
  else
    displayError='yes'
  fi

  if [ "$displayError" = 'no' ]
  then
    print "INFO: ($scriptName) - Valid NSM script package level found."
    print
  elif [ "$displayError" = 'yes' ]
  then
    writeErr "NSM-95102-E: ERROR($scriptName) - Minimum level of NSM script package not found!"
    writeErr "                                  Please take corrective action."
    writeErr
    eAndEnterToExit
  fi

  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting

}  # END verifyNsmPackageLevel

# ---------------------------------------------------------------------------
# Make sure /usr/nsm.3466 exists
# ---------------------------------------------------------------------------

function verifyNsmMainDir {

  subprocessName='verifyNsmMainDir'
  if [ "$skipVerifyNsmMainDir" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    if [ -d $nsmMainDir ]
    then
      print "INFO: ($scriptName) - Confirmed existence of $nsmMainDir."
      print
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      writeErr "NSM-95103-E: ERROR($scriptName) - $nsmMainDir does not exist!  Please call IBM support."
      writeErr
      eAndEnterToExit
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END verifyNsmMainDir

# ---------------------------------------------------------------------------
# Identify the NSM model
# ---------------------------------------------------------------------------

function determineModelNumber {

  subprocessName='determineModelNumber'
  if [ "$skipDetermineModelNumber" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    theFile="/usr/nsm.3466/sysInfo/3466.model"
    if [ -f $theFile ]
    then
      theModel=`cat $theFile | awk '{print $1}'`
      isItOld=`print $theModel | grep C`
      if [ "$isItOld" != "" ]
      then
        counter=1
        while [ "$char" != ',' ]
        do
          char=`print $theModel | cut -b$counter`
          (( counter = counter + 1 ))
          if [ "$char" != ',' ]
          then
            modelNumber=$modelNumber$char
          fi
        done
      else
        modelNumber=$theModel
      fi
    else
      modelNumber='???'
    fi

    print "INFO: ($scriptName) - Identified as a 3466 Model $modelNumber."
    print

    if [ "$modelNumber" = "C00" ] || [ "$modelNumber" = "C01" ]
    then
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      writeErr "NSM-95104-E: ERROR($scriptName) - You must have a C00 or C01 to proceed with this upgrade!"
      writeErr
      eAndEnterToExit
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END determineModelNumber

# ---------------------------------------------------------------------------
# Verify existence and validity of ADSM/TSM log directory
# ---------------------------------------------------------------------------

function verifyAdsmlogDir {

  subprocessName='verifyAdsmlogDir'
  if [ "$skipVerifyAdsmlogDir" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    if [[ -d $adsmlogDir ]]
    then
      print "INFO: ($scriptName) - Confirmed existence of $adsmlogDir."
      print
    else
      writeErr "NSM-95105-E: ERROR($scriptName) - $adsmlogDir does not exist!  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi
 
    # -------------------------------------------------------------------------
    # Make sure /adsmlog is mounted

    isItMounted=`df -k $adsmlogDir | grep $adsmlogDir`
    if [ "$isItMounted" != '' ]
    then
      print "INFO: ($scriptName) - $adsmlogDir is mounted."
      print
    else
      writeErr "NSM-95106-E: ERROR($scriptName) - $adsmlogDir is not mounted!  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    # -------------------------------------------------------------------------
    # Make sure /adsmlog has enough free space to save off files

    freeSpaceOfDir=`print $isItMounted | awk '{print $3}'`
    if (( $freeSpaceOfDir >= $minLogDirFreeSpace ))
    then
      print "INFO: ($scriptName) - $adsmlogDir contains sufficient space."
      print
      print "                      Current available free space: $freeSpaceOfDir kilobytes"
      print "                      Minimum required free space: $minLogDirFreeSpace kilobytes"
      print
    else
      writeErr "NSM-95107-E: ERROR($scriptName) - $adsmlogDir does not contain sufficient space!"
      writeErr
      writeErr "                                  Current detected 512-byte blocks free: $freeSpaceOfDir"
      writeErr "                                  Minimum required 512-byte blocks free: $minLogDirFreeSpace"
      writeErr
      writeErr "                                  Please extend the filesystem to at least the minimum required"
      writeErr "                                  size before continuing."
      writeErr
      eAndEnterToExit
    fi

    # -------------------------------------------------------------------------
    # Create the /adsmlog/prepare directory if not already there

    if [ -d "$prepareDir" ]
    then
      print "INFO: ($scriptName) - Found existing $prepareDir."
      print
      print "INFO: ($scriptName) - Renaming to $prepareDir.$timeStamp."
      print
      mv $prepareDir $prepareDir.$timeStamp
      mvRC=$?
      if [ "$mvRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr "NSM-95108-E: ERROR($scriptName) - Unable to rename $prepareDir to"
        writeErr "                                  $prepareDir.$timeStamp!  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    fi

    # -------------------------------------------------------------------------
    # Create the /adsmlog/prepare directory

    mkdir $prepareDir 2>/dev/null
    mkdirRC=$?
    if [ "$mkdirRC" = '0' ]
    then
      print "INFO: ($scriptName) - Successfully created $prepareDir."
      print
    else
      writeErr "NSM-95109-E: ERROR($scriptName) - Unable to create $prepareDir!  Please take corrective"
      writeErr "                                  action."
      writeErr
      eAndEnterToExit
    fi

    # -------------------------------------------------------------------------
    # Write a test file in the /adsmlog/prepare directory

    writeStatus=`print Testing 2>/dev/null > $prepareDir/testfile.$timeStamp`
    writeStatusRC=$?
    if [ "$writeStatusRC" = '0' ]
    then
      rm $prepareDir/testfile.$timeStamp
      print "INFO: ($scriptName) - Able to write to $prepareDir."
      print
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      writeErr "NSM-95110-E: ERROR($scriptName) - Unable to write to $prepareDir!  Please take corrective"
      writeErr "                                  action."
      writeErr
      eAndEnterToExit
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END verifyAdsmlogDir

# -----------------------------------------------------------------------------
# Check primary server status and start it if it's not already running
# -----------------------------------------------------------------------------

function startPrimaryServer {

  subprocessName='startPrimaryServer'
  if [ "$skipStartPrimaryServer" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    isitup $primaryServerName > /dev/null
    isitupRC=$?
    if [ "$isitupRC" = '0' ]
    then
      print "INFO: ($scriptName) - The TSM server on this system is running."
      print
      primaryServerStanzaName='ss-m'
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      print "INFO: ($scriptName) - The TSM server on this system is NOT running."
      print
      print "                      Issuing command: \"startadsm $primaryServerName\""
      print
      startadsm $primaryServerName
      startadsmRC=$?
      print
      if [ "$startadsmRC" = '0' ]
      then
        primaryServerStanzaName='ss-m'
        subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
      else
        writeErr "NSM-95111-E: ERROR($scriptName) - Could not start the TSM server.  Please take corrective action"
        writeErr "                                  and restart this script."
        writeErr
        eAndEnterToExit
      fi
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END startPrimaryServer

# -----------------------------------------------------------------------------
# Check primary server status and stop it if it's currently running
# -----------------------------------------------------------------------------

function stopPrimaryServer {

  subprocessName='stopPrimaryServer'
  if [ "$skipStopPrimaryServer" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    isitup $primaryServerName > /dev/null
    isitupRC=$?
    if [ "$isitupRC" = '1' ]
    then
      print "INFO: ($scriptName) - The TSM server on this system is NOT running."
      print
      primaryServerStanzaName='ss-m'
      subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
    else
      print "INFO: ($scriptName) - The TSM server on this system is running."
      print

      theCommand="$tsmInvoc halt"
      print "                      Issuing command: \"$theCommand\""
      print
      eval $theCommand > /dev/null
      stopadsmRC=$?
      sleepWithSpinner 10
      if [ "$stopadsmRC" = '0' ]
      then
        print "INFO: ($scriptName) - The TSM server has been halted."
        print
        primaryServerStanzaName='ss-m'
        subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
      else
        writeErr "NSM-95112-E: ERROR($scriptName) - Could not halt the TSM server.  Please take corrective action"
        writeErr "                                  and restart this script."
        writeErr
        eAndEnterToExit
      fi
    fi

    serverProcesses=`ps -eaf | grep dsmserv | grep -v grep | wc -l | awk '{print $1}'`
    while [ "$serverProcesses" != '0' ]
    do
      print "INFO: ($scriptName) - Other server processes are still detected.  This may be"
      print "                      the result of kin servers running on the system.  You"
      print "                      must halt these servers before continuing."
      print
      endadsm
      print
      serverProcesses=`ps -eaf | grep dsmserv | grep -v grep | wc -l | awk '{print $1}'`
    done
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END stopPrimaryServer

# ---------------------------------------------------------------------------
# Verify server communications
# ---------------------------------------------------------------------------

function verifyServerCommunications {

  subprocessName='verifyServerCommunications'
  if [ "$skipVerifyServerCommunications" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Issuing command: \". vadmin\""

    vadminRC=69
    while [ "$vadminRC" != '0' ]
    do
      . vadmin
      vadminRC=$?

      if [ "$vadminRC" != '0' ]
      then
        response=''
        until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
        do
          print
          read response?"QUERY: ($scriptName) - Do you wish to try again [y,n]? "
          print
        done

        if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
        then
          vadminRC=69
        else

          # ---------------------------------------------------------------------
          # Display error messages to STDERR

          writeErr ()
          {
            print "$*" >&2
          }

          writeErr "NSM-95113-E: ERROR($scriptName) - Unable to verify communications with the TSM server.  Please"
          writeErr "                                  correct the problem and restart the script."
          writeErr
          eAndEnterToExit
        fi
      fi
    done
    print

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END verifyServerCommunications

# ---------------------------------------------------------------------------
# Verify server communications automatically
# ---------------------------------------------------------------------------

function verifyServerCommAutomatically {

  subprocessName='verifyServerCommAutomatically'
  if [ "$skipVerifyServerCommAutomatically" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    startPrimaryServer

    vadminRC=69
    while [ "$vadminRC" != '0' ]
    do
      . vadmin -q $adsmID $adsmPSWD $adsmServer > /dev/null
      vadminRC=$?
      if [ "$vadminRC" != '0' ]
      then
        print "INFO: ($scriptName) - Server is not up yet."
        print
        sleepWithSpinner 60
      fi
    done

    subprocessName='verifyServerCommAutomatically'
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END verifyServerCommAutomatically

# ---------------------------------------------------------------------------
# Assign TSM userid and password variables
# ---------------------------------------------------------------------------

function assignTsmAdminVariables {

  subprocessName='assignTsmAdminVariables'
  if [ "$skipAssignTsmAdminVariables" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    # ---------------------------------------------------------------------------
    # Assign TSM admin ID and PASSWORD values (from vadmin)
    # Exported values used: adminUserID, adminPassword and serverStanza

    if [ "$adminUserID" = '' ]
    then
      adsmID='admin'
    else
      adsmID=$adminUserID
    fi

    if [ "$adminPassword" = '' ]
    then
      adsmPSWD='admin'
    else
      adsmPSWD=$adminPassword
    fi

    if [ "$serverStanza" = '' ]
    then
      adsmServer='ss-m'
    else
      adsmServer=$serverStanza
    fi

    if [ "$xa" = '' ]
    then
      tsmInvoc="dsmadmc -i -id=$adsmID -pa=$adsmPSWD -server=$adsmServer"
    else
      tsmInvoc="$xa"  # Rename exported value from vadmin
    fi

    export adsmID=$adsmID
    export adminUserID=$adsmID
    export adsmPSWD=$adsmPSWD
    export adminPassword=$adsmPSWD
    export adsmServer=$adsmServer
    export serverStanza=$adsmServer

    if [ "$currentPhase" = '1' ]
    then
      print "TSMADMINID $adsmID" >> $prepareDir/$criticalSysInfo
      print >> $prepareDir/$criticalSysInfo
      print "TSMADMINPASSWORD $adsmPSWD" >> $prepareDir/$criticalSysInfo
      print >> $prepareDir/$criticalSysInfo
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END assignTsmAdminVariables

# ---------------------------------------------------------------------------
# Verify minimum level of TSM exists
# ---------------------------------------------------------------------------

function verifyApprovedTsmLevel {

  subprocessName='verifyApprovedTsmLevel'
  if [ "$skipVerifyApprovedTsmLevel" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    # -------------------------------------------------------------------------
    # Query and save TSM version and release numbers

    tsmServerVersion=`$tsmInvoc q st | grep 'Server Version'`

    currentTsmVersion=`print $tsmServerVersion | awk '{print $3}' | cut -c1`
    currentTsmRelease=`print $tsmServerVersion | awk '{print $5}' | cut -c1`
    currentTsmLevel=`print $tsmServerVersion | awk '{print $7}'`

    print "INFO: ($scriptName) - This system is running TSM Server $currentTsmVersion.$currentTsmRelease.$currentTsmLevel."
    print

    # ---------------------------------------------------------------------
    # Display error messages to STDERR

    writeErr ()
    {
      print "$*" >&2
    }

    if [ "$currentTsmVersion" -lt "$approvedTsmVersion" ]
    then
      writeErr "NSM-95114-E: ERROR($scriptName) - You do not have the pre-requisite version of TSM required to load"
      writeErr "                                  this EC.  $approvedTsmVersion.$approvedTsmRelease.x.x must be installed before this script will"
      writeErr "                                  continue."
      writeErr
      eAndEnterToExit
    elif [ "$currentTsmVersion" -gt "$approvedTsmVersion" ]
    then
      writeErr "NSM-95115-E: ERROR($scriptName) - You have a newer version of TSM than is expected to load this EC."
      writeErr "                                  This is an unsupported condition.  Please contact IBM support for"
      writeErr "                                  further assistance."
      writeErr
      eAndEnterToExit
    elif [ "$currentTsmVersion" = "$completedTsmVersion" ] && [ "$currentTsmRelease" = "$completedTsmRelease" ]
    then
      writeErr "NSM-95116-E: ERROR($scriptName) - You have the same version and release of TSM that is to be loaded"
      writeErr "                                  by this EC.  This is an unsupported condition.  Please contact"
      writeErr "                                  IBM support for further assistance."
      writeErr
      eAndEnterToExit
    elif [ "$currentTsmVersion" = "$completedTsmVersion" ] && [ "$currentTsmRelease" -gt "$completedTsmRelease" ]
    then
      writeErr "NSM-95117-E: ERROR($scriptName) - You have a newer release of TSM than is expected to load this EC."
      writeErr "                                  This is an unsupported condition.  Please contact IBM support for"
      writeErr "                                  further assistance."
      writeErr
      eAndEnterToExit
    elif [ "$currentTsmVersion" != "$approvedTsmVersion" ] || [ "$currentTsmRelease" != "$approvedTsmRelease" ]
    then
      writeErr "NSM-95118-E: ERROR($scriptName) - You do not have the pre-requisite level of TSM required to load"
      writeErr "                                  this EC.  $approvedTsmVersion.$approvedTsmRelease.x.x must be installed before this script will"
      writeErr "                                  continue."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END verifyApprovedTsmLevel

# ---------------------------------------------------------------------------
# Run verification scripts
# ---------------------------------------------------------------------------

function callVerify3466Scripts {

  subprocessName='callVerify3466Scripts'
  if [ "$skipCallVerify3466Scripts" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    function runVerification {
      print "INFO: ($scriptName) - Issuing command: \"verify3466ssa -n\""
      verify3466ssa -n
      verify3466ssaRC=$?
      if [ "$verify3466ssaRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr "NSM-95119-E: ERROR($scriptName) - verify3466ssa did not complete successfully!  Please correct any"
        writeErr "                                  problems identified before restarting this script."
        writeErr
        eAndEnterToExit
      fi

      verify3466tsm -n
      verify3466tsmRC=$?
      if [ "$verify3466tsmRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr "NSM-95120-E: ERROR($scriptName) - verify3466tsm did not complete successfully!  Please correct any"
        writeErr "                                  problems identified before restarting this script."
        writeErr
        eAndEnterToExit
      fi
    }  # END runVerification

    if [ -d "$prepareDir.$timeStamp" ]
    then
      alreadyExecuted=`cat $prepareDir.$timeStamp/$criticalSysInfo | grep VERIFY3466SCRIPTS | awk '{print $2}'`
    else
      alreadyExecuted=''
    fi

    if [ "$alreadyExecuted" = '' ]
    then
      runVerification
    else
      response=''
      until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
      do
        print "INFO: ($scriptName) - The 3466 verification scripts were detected to have"
        print "                      been previously executed.  If no critical hardware or"
        print "                      software changes have been made since then, you may"
        print "                      elect to bypass executing them again."
        print
        read response?"                      Do you wish to run them again [y,n]? "
        print
      done

      if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
      then
        print "INFO: ($scriptName) - User selected [$response] -- Re-execute verification scripts."
        print
        runVerification
      else
        print "INFO: ($scriptName) - User selected [$response] -- Bypass verification scripts."
        print
      fi
    fi

    print "VERIFY3466SCRIPTS Successful" >> $prepareDir/$criticalSysInfo
    print >> $prepareDir/$criticalSysInfo

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END callVerify3466Scripts

# ---------------------------------------------------------------------------
# Collect generic system information
# ---------------------------------------------------------------------------

function gatherSysInfo {

  subprocessName='gatherSysInfo'
  if [ "$skipGatherSysInfo" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Collecting generic system information ..."
    print "                      (This may take a few minutes)"
    print

    somethingFailed='no'
    print "# ---------------------------------------------------------------------------" > $prepareDir/$genericSysInfo
    print "# Generic System Information for $hostName" >> $prepareDir/$genericSysInfo
    print "# ---------------------------------------------------------------------------" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="date"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="df"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg rootvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg -p rootvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg -l rootvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg adsmvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg -p adsmvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo
    print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    theCommand="lsvg -l adsmvg"
    print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
    print >> $prepareDir/$genericSysInfo
    eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingFailed='yes'
    fi
    print >> $prepareDir/$genericSysInfo

    # -------------------------------------------------------------------------
    # Only check the "adsmvg2" volume group if more than 32 SSA disks are
    # detected

    diskCount=$(lsdev -Cc disk | grep -c SSA)
    if [ "$diskCount" -gt 32 ]
    then
      print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      theCommand="lsvg adsmvg2"
      print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
      evalRC=$?
      if [ "$evalRC" != '0' ]
      then
        somethingFailed='yes'
      fi
      print >> $prepareDir/$genericSysInfo
      print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      theCommand="lsvg -p adsmvg2"
      print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
      evalRC=$?
      if [ "$evalRC" != '0' ]
      then
        somethingFailed='yes'
      fi
      print >> $prepareDir/$genericSysInfo
      print "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      theCommand="lsvg -l adsmvg2"
      print "Command issued: $theCommand" >> $prepareDir/$genericSysInfo
      print >> $prepareDir/$genericSysInfo
      eval $theCommand 2>/dev/null >> $prepareDir/$genericSysInfo
      evalRC=$?
      if [ "$evalRC" != '0' ]
      then
        somethingFailed='yes'
      fi
      print >> $prepareDir/$genericSysInfo
    fi

    print "# ---------------------------------------------------------------------------" >> $prepareDir/$genericSysInfo

    if [ "$somethingFailed" = 'no' ]
    then
      print "INFO: ($scriptName) - Information successfully collected in:"
      print "                      $prepareDir/$genericSysInfo"
      print
    else
      writeErr "NSM-95121-E: ERROR($scriptName) - One or more query commands failed.  Please check the"
      writeErr "                                  $prepareDir/$genericSysInfo file for more information."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    genericSysInfo=""
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END gatherSysInfo

# ---------------------------------------------------------------------------
# Create critical system info file header
# ---------------------------------------------------------------------------

function createCriticalSysInfoHeader {

    print "# ---------------------------------------------------------------------------" > $prepareDir/$criticalSysInfo
    print "# Critical System Information for $hostName" >> $prepareDir/$criticalSysInfo
    print "# ---------------------------------------------------------------------------" >> $prepareDir/$criticalSysInfo
    print >> $prepareDir/$criticalSysInfo

}  # END createCriticalSysInfoHeader

# ---------------------------------------------------------------------------
# Save time zone information
# ---------------------------------------------------------------------------

function saveTimeZone {

  subprocessName='saveTimeZone'
  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

  timeZone=$(date | awk '{print $5}' | tr '[a-z]' '[A-Z]' | cut -b1-3)

  print "INFO: ($scriptName) - Time zone identified as $timeZone."
  print
  print "TIMEZONE  $timeZone" >> $prepareDir/$criticalSysInfo
  print >> $prepareDir/$criticalSysInfo

  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting

}  # END saveTimeZone

# ---------------------------------------------------------------------------
# Check for full filesystems in rootvg
# ---------------------------------------------------------------------------

function checkRootVgFilesystems {

  subprocessName='checkRootVgFilesystems'
  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

  print "INFO: ($scriptName) - Checking rootvg filesystem usage ..."
  print
  print "                      Filesystem  % Used"
  print "                      ----------  ------"

  # More flexible solution if needed in the future
  # rootvgFs=$(lsvg -l rootvg | awk '{print $7}' | grep -v 'LV' | grep -v 'N/A')

  rootvgFs="/ /home /tmp /usr /var"
  listOfFullFilesystems=''
  for eachFs in $rootvgFs
  do
    fsSize=$(df $eachFs | grep $eachFs | awk '{print $4}')

    fsChars=$(print $eachFs | wc -c | awk '{print $1}')
    let fsChars=fsChars-1

    fsUseChars=$(print $fsSize | wc -c | awk '{print $1}')
    let fsUseChars=fsUseChars-1

    # -----------------------------------------------------------------------
    # Need to add more char support for fs names longer than 5 chars

    if [ "$fsChars" = '5' ]
    then
      if [ "$fsUseChars" = '4' ]
      then
        print "                      $eachFs        $fsSize"
      elif [ "$fsUseChars" = '3' ]
      then
        print "                      $eachFs         $fsSize"
      elif [ "$fsUseChars" = '2' ]
      then
        print "                      $eachFs          $fsSize"
      fi
    elif [ "$fsChars" = '4' ]
    then
      if [ "$fsUseChars" = '4' ]
      then
        print "                      $eachFs         $fsSize"
      elif [ "$fsUseChars" = '3' ]
      then
        print "                      $eachFs          $fsSize"
      elif [ "$fsUseChars" = '2' ]
      then
        print "                      $eachFs           $fsSize"
      fi
    elif [ "$fsChars" = '3' ]
    then
      if [ "$fsUseChars" = '4' ]
      then
        print "                      $eachFs          $fsSize"
      elif [ "$fsUseChars" = '3' ]
      then
        print "                      $eachFs           $fsSize"
      elif [ "$fsUseChars" = '2' ]
      then
        print "                      $eachFs            $fsSize"
      fi
    elif [ "$fsChars" = '2' ]
    then
      if [ "$fsUseChars" = '4' ]
      then
        print "                      $eachFs           $fsSize"
      elif [ "$fsUseChars" = '3' ]
      then
        print "                      $eachFs            $fsSize"
      elif [ "$fsUseChars" = '2' ]
      then
        print "                      $eachFs             $fsSize"
      fi
    elif [ "$fsChars" = '1' ]
    then
      if [ "$fsUseChars" = '4' ]
      then
        print "                      $eachFs            $fsSize"
      elif [ "$fsUseChars" = '3' ]
      then
        print "                      $eachFs             $fsSize"
      elif [ "$fsUseChars" = '2' ]
      then
        print "                      $eachFs              $fsSize"
      fi
    fi

    if [ "$fsSize" = '100%' ]
    then
      listOfFullFilesystems="$listOfFullFilesystems $eachFs"
    fi
  done

  print
  numOfFullFs=$(print $listOfFullFilesystems | wc -w | awk '{print $1}')
  if [ "$listOfFullFilesystems" = '' ]
  then
    :  # Proceed normally
  else
    if [ "$numOfFullFs" = '1' ]
    then
      writeErr "NSM-95122-E: ERROR($scriptName) - The following filesystem is 100% full:"
    else
      writeErr "NSM-95122-E: ERROR($scriptName) - The following filesystems are 100% full:"
    fi 
    writeErr
    for eachFullFs in $listOfFullFilesystems
    do
      writeErr "                                  $eachFullFs"
    done
    writeErr
    if [ "$numOfFullFs" = '1' ]
    then
      writeErr "                                  Please remove unnecessary files or expand its size before"
    else
      writeErr "                                  Please remove unnecessary files or expand their sizes before"
    fi
    writeErr "                                  restarting this script."
    writeErr
    eAndEnterToExit
  fi

  subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting

}  # END checkRootVgFilesystems

# ---------------------------------------------------------------------------
# Checks to make sure a full TSM database backup was done
# ---------------------------------------------------------------------------

function checkForFullDatabaseBackup {

  subprocessName='checkForFullDatabaseBackup'
  if [ "$skipCheckForFullDatabaseBackup" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Checking for a full TSM database backup from today ..."
    print

    eval "$tsmInvoc q volh type=dbbackup begindate=today-1 begintime=now > /tmp/hold.$timeStamp"
    backupSequence=$(cat /tmp/hold.$timeStamp | grep -c "BACKUPFULL")
    backupSeries=$(cat /tmp/hold.$timeStamp | grep "BACKUPFULL" | grep -n "BACKUPFULL" | grep ^$backupSequence | awk '{print $4}')

    rm /tmp/hold.$timeStamp

    if [ "$backupSeries" = '' ]
    then
      writeErr "NSM-95123-E: ERROR($scriptName) - Unable to find a full TSM database backup"
      writeErr "                                  from within the last 24 hours.  Please take"
      writeErr "                                  corrective action."
      writeErr
      eAndEnterToExit
    else
      print "INFO: ($scriptName) - A full TSM database backup from within the last 24 hours"
      print "                      (Series: $backupSeries) was detected."
      print
      print "FULLDBBACKUPSERIES  $backupSeries" >> $prepareDir/$criticalSysInfo
      print >> $prepareDir/$criticalSysInfo
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END checkForFullDatabaseBackup

# ---------------------------------------------------------------------------
# Collect TSM server licensing information
# ---------------------------------------------------------------------------

function collectTsmLicenseInformation {

  subprocessName='collectTsmLicenseInformation'
  if [ "$skipCollectTsmLicenseInformation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    licensesCollected=''
    licenseCompliance=`$tsmInvoc -commadelimit select "'keyID'",COMPLIANCE from licenses | grep 'keyID' | grep -v 'select' | cut -f2- -d','`

    if [ "$licenseCompliance" = 'Valid' ]
    then
      licensesToCheck="MGSYSLAN_LIC SPACEMAN_LIC DRM_LIC MGSYSSAN_LIC ORACLE_LIC MSSQL_LIC MSEXCH_LIC LNOTES_LIC DOMINO_LIC INFORMIX_LIC SAPR3_LIC ESS_LIC ESSR3_LIC EMCSYMM_LIC EMCSYMR3_LIC LIBSHARE_LIC LIBRARY_LIC"

      for eachLicense in $licensesToCheck
      do
        eachLicenseValue=`$tsmInvoc -commadelimit select "'keyID'",$eachLicense from licenses | grep 'keyID' | grep -v 'select' | cut -f2- -d','`

        licensesCollected="$licensesCollected $eachLicense,$eachLicenseValue"
      done

      for eachLicenseBundle in $licensesCollected
      do
        licenseName=`print $eachLicenseBundle | cut -f1 -d','`
        licenseValue=`print $eachLicenseBundle | cut -f2 -d','`

        if [ "$licenseName" = 'MGSYSLAN_LIC' ]
        then
          print "                         Number of Managed System for LAN licensed: $licenseValue"
        elif [ "$licenseName" = 'SPACEMAN_LIC' ]
        then
          print "                       Number of space management clients licensed: $licenseValue"
        elif [ "$licenseName" = 'DRM_LIC' ]
        then
          print "                       Tivoli Disaster Recovery Manager licensed ?: $licenseValue"
        elif [ "$licenseName" = 'MGSYSSAN_LIC' ]
        then
          print "                         Number of Managed System for SAN licensed: $licenseValue"
        elif [ "$licenseName" = 'ORACLE_LIC' ]
        then
          print "                                 Number of TDP for Oracle licensed: $licenseValue"
        elif [ "$licenseName" = 'MSSQL_LIC' ]
        then
          print "                          Number of TDP for MS SQL Server licensed: $licenseValue"
        elif [ "$licenseName" = 'MSEXCH_LIC' ]
        then
          print "                            Number of TDP for MS Exchange licensed: $licenseValue"
        elif [ "$licenseName" = 'LNOTES_LIC' ]
        then
          print "                            Number of TDP for Lotus Notes licensed: $licenseValue"
        elif [ "$licenseName" = 'DOMINO_LIC' ]
        then
          print "                           Number of TDP for Lotus Domino licensed: $licenseValue"
        elif [ "$licenseName" = 'INFORMIX_LIC' ]
        then
          print "                               Number of TDP for Informix licensed: $licenseValue"
        elif [ "$licenseName" = 'SAPR3_LIC' ]
        then
          print "                                Number of TDP for SAP R/3 licensed: $licenseValue"
        elif [ "$licenseName" = 'ESS_LIC' ]
        then
          print "                                    Number of TDP for ESS licensed: $licenseValue"
        elif [ "$licenseName" = 'ESSR3_LIC' ]
        then
          print "                                Number of TDP for ESS R/3 licensed: $licenseValue"
        elif [ "$licenseName" = 'EMCSYMM_LIC' ]
        then
          print "                          Number of TDP for EMC Symmetrix licensed: $licenseValue"
        elif [ "$licenseName" = 'EMCSYMR3_LIC' ]
        then
          print "                      Number of TDP for EMC Symmetrix R/3 licensed: $licenseValue"
        elif [ "$licenseName" = 'LIBSHARE_LIC' ]
        then
          print "                                       Is Library Sharing licensed: $licenseValue"
        elif [ "$licenseName" = 'LIBRARY_LIC' ]
        then
          print "                              Number of Managed Libraries licensed: $licenseValue"
        fi
      done
      print

      print "TSM41LICENSES $licensesCollected" >> $prepareDir/$criticalSysInfo
      print >> $prepareDir/$criticalSysInfo
    else
      writeErr "NSM-95124-E: ERROR($scriptName) - TSM server licenses are not in compliance!"
      writeErr "                                  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END collectTsmLicenseInformation

# ---------------------------------------------------------------------------
# Disable migration for all TSM storage pools
# ---------------------------------------------------------------------------

function disableMigration {

  subprocessName='disableMigration'
  if [ "$skipDisableMigration" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Disabling TSM storage pool migration ..."
    print

    nextStgpoolDetected='no'
    stgpoolsProcessed=''

    listOfStgpools=`$tsmInvoc -commadelimit select "'keyID'",STGPOOL_NAME,NEXTSTGPOOL from stgpools | grep 'keyID' | grep -v 'select' | cut -f2- -d','`

    for eachStgpoolBundle in $listOfStgpools
    do
      stgpool=$(print $eachStgpoolBundle | cut -f1 -d',')
      nextStgpool=$(print $eachStgpoolBundle | cut -f2 -d',')
      if [ "$nextStgpool" = '' ]
      then
        nextStgpool='None'
      else
        nextStgpoolDetected='yes'
      fi

      if [ "$nextStgpool" = 'None' ]
      then
        print "                      $stgpool -> $nextStgpool"
        stgpoolsProcessed="$stgpoolsProcessed $stgpool,$nextStgpool "
      else
        $tsmInvoc "upd stg $stgpool next=''" > /dev/null
        disableTsmMigRC=$?
        if [ "$disableTsmMigRC" = '0' ]
        then
          print "                      $stgpool -> $nextStgpool (Disabled)"
          stgpoolsProcessed="$stgpoolsProcessed $stgpool,$nextStgpool "
          tsmMigrationDisabled='yes'
        else
          writeErr "NSM-95125-E: ERROR($scriptName) - Unable to set next storage pool for $stgpool to null!"
          writeErr "                                  Please take corrective action."
          writeErr
          eAndEnterToExit
        fi
      fi
    done
    print

    print "TSMSTGPOOLMIG $stgpoolsProcessed" >> $prepareDir/$criticalSysInfo
    print >> $prepareDir/$criticalSysInfo

    if [[ $nextStgpoolDetected = 'yes' ]]
    then
      print "TSMSTGPOOLMIG $stgpoolsProcessed" > $adsmlogDir/$storagePoolMigration
    fi

    if [ "$tsmMigrationDisabled" = 'yes' ]
    then
      print "INFO: ($scriptName) - TSM storage pool migration successfully disabled."
      print
    else
      print "INFO: ($scriptName) - TSM storage pool migration is not enabled."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END disableMigration

# ---------------------------------------------------------------------------
# Enable migration for all disk storage pools
# ---------------------------------------------------------------------------

function enableMigration {

  subprocessName='enableMigration'
  if [ "$skipEnableMigration" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Enabling TSM storage pool migration ..."
    print

    if [[ $whichPhase = 'phase1' ]]
    then
      stgpoolBundles=`cat $adsmlogDir/$storagePoolMigration | grep TSMSTGPOOLMIG | cut -f2- -d' '`

    elif [[ $whichPhase = 'phase2' ]]
    then
      stgpoolBundles=`cat $restoreDir/$criticalSysInfo | grep TSMSTGPOOLMIG | cut -f2- -d' '`
    fi

    if [ "$stgpoolBundles" = '' ]
    then
      print "INFO: ($scriptName) - TSM storage pool migration is not enabled."
      print
    else
      for eachStgpoolBundle in $stgpoolBundles
      do
        stgpoolName=`print $eachStgpoolBundle | cut -f1 -d','`
        nextStgpoolName=`print $eachStgpoolBundle | cut -f2 -d','`

        if [ "$nextStgpoolName" = 'None' ]
        then
          enableTsmMigRC=0
        else
          $tsmInvoc upd stg $stgpoolName next=$nextStgpoolName > /dev/null
          enableTsmMigRC=$?
        fi

        if [ "$enableTsmMigRC" = '0' ]
        then
          print "                      $stgpoolName -> $nextStgpoolName"
        else
          writeErr "NSM-95126-E: ERROR($scriptName) - Unable to set next storage pool for $stgpoolName to $nextStgpoolName!"
          writeErr "                                  Please take corrective action."
          writeErr
          eAndEnterToExit
        fi
      done
      print
      print "INFO: ($scriptName) - TSM storage pool migration successfully enabled."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END enableMigration

# ---------------------------------------------------------------------------
# Disable reclamation for all primary sequential access storage pools
# ---------------------------------------------------------------------------

function disableReclamation {

  subprocessName='disableReclamation'
  if [ "$skipDisableReclamation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Disabling TSM primary sequential access storage pool"
    print "                      reclamation ..."
    print

    stgpoolsProcessed=''
    listOfStgpools=`$tsmInvoc -commadelimit select "'keyID'",STGPOOL_NAME from stgpools where devclass!="'DISK'" | grep 'keyID' | grep -v 'select' | cut -f2 -d','`
    if [ "$listOfStgpools" = '' ]
    then
      print "INFO: ($scriptName) - No TSM primary sequential access storage pools found."
      print
    else
      for eachStgpool in $listOfStgpools
      do
        reclamationThreshold=`$tsmInvoc -commadelimit select "'keyID'",RECLAIM from stgpools where stgpool_name="'$eachStgpool'" | grep 'keyID' | grep -v 'select' | cut -f2 -d','`
        if [ "$reclamationThreshold" = '100' ]
        then
          print "                      $eachStgpool -- $reclamationThreshold% reclamation"
          stgpoolsProcessed="$stgpoolsProcessed $eachStgpool,$reclamationThreshold "
        else
          $tsmInvoc upd stg $eachStgpool reclaim=100 > /dev/null
          disableTsmRecRC=$?
          if [ "$disableTsmRecRC" = '0' ]
          then
            print "                      $eachStgpool -- $reclamationThreshold% reclamation (Disabled)"
            stgpoolsProcessed="$stgpoolsProcessed $eachStgpool,$reclamationThreshold "
            tsmReclamationDisabled='yes'
          else
            writeErr
            writeErr "NSM-95127-E: ERROR($scriptName) - Unable to set reclamation for storage pool $stgpool to 100%!"
            writeErr "                                  Please take corrective action."
            writeErr
            eAndEnterToExit
          fi
        fi
      done

      print "TSMSTGPOOLREC $stgpoolsProcessed" >> $prepareDir/$criticalSysInfo
      print >> $prepareDir/$criticalSysInfo

      print

      if [ "$tsmReclamationDisabled" = 'yes' ]
      then
        print "INFO: ($scriptName) - TSM primary sequential access storage pool reclamation"
        print "                      successfully disabled."
        print
      else
        print "INFO: ($scriptName) - TSM primary sequential access storage pool reclamation"
        print "                      is not enabled."
        print
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END disableReclamation

# ---------------------------------------------------------------------------
# Enable reclamation for all primary sequential access storage pools
# ---------------------------------------------------------------------------

function enableReclamation {

  subprocessName='enableReclamation'
  if [ "$skipEnableReclamation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Enabling TSM primary sequential access storage pool"
    print "                      reclamation ..."
    print

    stgpoolBundles=`cat $restoreDir/$criticalSysInfo | grep TSMSTGPOOLREC | cut -f2- -d' '`
    if [ "$stgpoolBundles" = '' ]
    then
      print "INFO: ($scriptName) - TSM primary sequential access storage pool reclamation"
      print "                      is not enabled."
      print
    else
      for eachStgpoolBundle in $stgpoolBundles
      do
        stgpoolName=`print $eachStgpoolBundle | cut -f1 -d','`
        stgpoolReclamationThreshold=`print $eachStgpoolBundle | cut -f2 -d','`

        $tsmInvoc upd stg $stgpoolName reclaim=$stgpoolReclamationThreshold > /dev/null
        enableTsmRecRC=$?
        if [ "$enableTsmRecRC" = '0' ]
        then
          print "                      $stgpoolName -- $stgpoolReclamationThreshold% reclamation (Enabled)"
        else
          writeErr
          writeErr "NSM-95128-E: ERROR($scriptName) - Unable to set reclamation for storage pool $stgpoolName to $stgpoolReclamationThreshold%!"
          writeErr "                                  Please take corrective action."
          writeErr
          eAndEnterToExit
        fi
      done
      print
      print "INFO: ($scriptName) - TSM primary sequential access storage pool reclamation"
      print "                      successfully enabled."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END enableReclamation

# ---------------------------------------------------------------------------
# Disable TSM server schedules
# ---------------------------------------------------------------------------

function disableSchedules {

  subprocessName='disableSchedules'
  if [ "$skipDisableSchedules" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Disabling TSM server schedules ..."
    print

    alreadyDisabled=$($tsmInvoc q opt | grep 'DisableScheds' | awk '{print $2}')
    if [ "$alreadyDisabled" = 'Yes' ]
    then
      print "INFO: ($scriptName) - TSM server schedules already disabled."
      print
    else
      stopPrimaryServer

      print "INFO: ($scriptName) - Adding \"DISABLESCheds Yes\" to dsmserv.opt file."
      print
      print >> /usr/tivoli/tsm/server/bin/dsmserv.opt
      print "    DISABLESCheds Yes" >> /usr/tivoli/tsm/server/bin/dsmserv.opt
      addRC=$?
      if [ "$addRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr "NSM-95129-E: ERROR($scriptName) - Unable to add \"DISABLESCheds Yes\" to dsmserv.opt file!"
        writeErr "                                  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi

      verifyServerCommAutomatically

      print "INFO: ($scriptName) - TSM server schedules successfully disabled."
      print
    fi

    subprocessName='disableSchedules'
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END disableSchedules

# ---------------------------------------------------------------------------
# Disable TSM client sessions
# ---------------------------------------------------------------------------

function disableSessions {

  subprocessName='disableSessions'
  if [ "$skipDisableSessions" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Disabling TSM client sessions ..."
    print

    sessionStatus=`$tsmInvoc select "'keyID'",AVAILABILITY from status | grep 'keyID' | grep -v 'select' | awk '{print $2}'`
    if [ "$sessionStatus" = 'Disabled' ]
    then
      print "INFO: ($scriptName) - TSM client sessions already disabled."
      print
    else
      $tsmInvoc disa sess > /dev/null
      disableRC=$?
      if [ "$disableRC" = '0' ]
      then
        print "INFO: ($scriptName) - TSM client sessions successfully disabled."
        print
      else
        writeErr "NSM-95130-E: ERROR($scriptName) - Unable to disable TSM client sessions.  Please take corrective"
        writeErr "                                  action and restart this script."
        writeErr
        eAndEnterToExit
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END disableSessions

# ---------------------------------------------------------------------------
# Collect all the tape information on the system
# ---------------------------------------------------------------------------

function collectTapeInformation {

  subprocessName='collectTapeInformation'
  if [ "$skipCollectTapeInformation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Collecting tape information ..."
    print

    function displayMenu {
      print "[1] Yes, I understand and have collected the necessary information.  Continue."
      print
      print "[2] No, I want to exit this script."
      print
    }  # END displayMenu

    function userSelect {
      displayHeader
      displayMenu
      response=0
      until [ "$response" = '1' ] || [ "$response" = '2' ]
      do
        read response?"Please make your selection [1,2]: "
        if [ "$response" = '1' ] || [ "$response" = '2' ]
        then
          : # Pass through
        else
          displayHeader
          print "-> $response <- is an invalid response!  Please try again."
          print
          print
          displayMenu
        fi
      done

      case $response in
        (1)
          print
          print "INFO: ($scriptName) - User selected Option [1] -- Continuing script."
          print
          ;;
        (2)
          print
          print "INFO: ($scriptName) - User selected Option [2] -- Exiting script."
          print
          programRC=1
          exit $programRC
          ;;
      esac
    }  # END userSelect

    if [ "$theMachineType" = 'H70' ] && [ "$modelNumber" = 'C01' ]
    then
      function displayHeader {
        print
        print "+-----------------------------------------------------------------------------+"
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|  Tape information will be automatically collected for IBM 3494, 3575,       |"
        print "|  3583 and 3584 libraries ONLY!  If there are any attached OEM libraries,    |"
        print "|  you must manually record information about those devices from AIX and TSM  |"
        print "|  prior to performing the mksysb restore process.  Failure to do so may      |"
        print "|  prevent TSM from functioning properly when the restore is complete.        |"
        print "|                                                                             |"
        print "|  You may open another AIX terminal window to collect the required data,     |"
        print "|  or exit this script and re-start it when you have the necessary            |"
        print "|  information.                                                               |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "+-----------------------------------------------------------------------------+"
        print
        print
      }  # END displayHeader

      userSelect

      theCommand="getTapeInfo -q -e $prepareDir $getTapeInfoBeforeFile"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
    else
      function displayHeader {
        print
        print "+-----------------------------------------------------------------------------+"
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|  Tape information will NOT be collected automatically because this is a     |"
        print "|  $modelNumber.  You must manually record current device information from AIX and     |"
        print "|  TSM prior to performing the mksysb restore process.  Failure to do so may  |"
        print "|  prevent TSM from functioning properly when the restore is complete.        |"
        print "|                                                                             |"
        print "|  You may open another AIX terminal window to collect the required data,     |"
        print "|  or exit this script and re-start it when you have the necessary            |"
        print "|  information.                                                               |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "+-----------------------------------------------------------------------------+"
        print
        print
      }  # END displayHeader

      getTapeInfoBeforeFile=''
      userSelect

    fi
    print

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    getTapeInfoBeforeFile=''
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END collectTapeInformation

# ---------------------------------------------------------------------------
# Collect network information
# ---------------------------------------------------------------------------

function collectNetworkInformation {

  subprocessName='collectNetworkInformation'
  if [ "$skipCollectNetworkInformation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - System network information will now be saved."
    print
    cAndEnterToContinue

    theCommand="captureIPenvironment $prepareDir/$captureIPenvironmentFile"
    print "                      Issuing command: \"$theCommand\""
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print "INFO: ($scriptName) - Successfully saved system IP information."
      print
    else
      writeErr "NSM-95131-E: ERROR($scriptName) - Unable to save system IP information!  Please take"
      writeErr "                                  corrective action."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    captureIPenvironmentFile=""
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END collectNetworkInformation

# ---------------------------------------------------------------------------
# Check file sizes for TSM server accounting, fileexit and userexit logs
# ---------------------------------------------------------------------------

function checkMiscTsmServerFileSizes {

  subprocessName='checkMiscTsmServerFileSizes'
  if [ "$skipCheckMiscTsmServerFileSizes" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    totalFileSizes=''
    fileSizeA=''
    fileSizeB=''
    fileSizeC=''

    print "INFO: ($scriptName) - Checking TSM server accounting, fileexit and userexit"
    print "                      file sizes ..."
    print

    fileA="dsmaccnt.log"

    if [ -f "$pathToTsmServerFiles/$fileA" ]
    then
      fileSizeA=`ls -l $pathToTsmServerFiles/$fileA | awk '{print $5}'`
      let fileSizeA=$fileSizeA/1024
      let fileSizeA=$fileSizeA+1
    else
      print "                      TSM server accounting file ($fileA) not found."
      print "                      This may be a valid condition."
      print
    fi

    fileB=`cat $pathToTsmServerFiles/dsmserv.opt | grep -i fileexit | awk '{print $3}'`

    if [ -f "$fileB" ]
    then
      fileSizeB=`ls -l $fileB | awk '{print $5}'`
      let fileSizeB=$fileSizeB/1024
      let fileSizeB=$fileSizeB+1
    else
      if [ "$fileB" = '' ]
      then
        fileB='?'
      fi
      print "                      TSM fileexit file ($fileB) not found."
      print "                      This may be a valid condition."
      print
    fi

    fileC=`cat $pathToTsmServerFiles/dsmserv.opt | grep -i userexit | awk '{print $3}'`

    if [ -f "$fileC" ]
    then
      fileSizeC=`ls -l $fileC | awk '{print $5}'`
      let fileSizeC=$fileSizeC/1024
      let fileSizeC=$fileSizeC+1
    else
      if [ "$fileC" = '' ]
      then
        fileC='?'
      fi
      print "                      TSM userexit file ($fileC) not found."
      print "                      This may be a valid condition."
      print
    fi

    if [ "$fileSizeA" = '' ] && [ "$fileSizeB" = '' ] && [ "$fileSizeC" = '' ]
    then
      print "INFO: ($scriptName) - No TSM server accounting, fileexit or userexit files"
      print "                      found."
      print
      miscTsmFiles='no'
    else
      print "INFO: ($scriptName) - File summary:"
      print "                      -------------"
      if [ "$fileSizeA" = '' ]
      then
        fileSizeA=0
      else
        print "                      $pathToTsmServerFiles/$fileA -- $fileSizeA kilobytes"
      fi
      if [ "$fileSizeB" = '' ]
      then
        fileSizeB=0
      else
        print "                      $fileB -- $fileSizeB kilobytes"
      fi
      if [ "$fileSizeC" = '' ]
      then
        fileSizeC=0
      else
        print "                      $fileC -- $filesizeC kilobytes"
      fi
      print

      let totalFileSizes=$fileSizeA+$fileSizeB+$fileSizeC
      print "                      Total size of files = $totalFileSizes kilobytes"
      print
      if [ "$totalFileSize" -lt "$freeSpaceOfDir" ]
      then
        print "                      There is sufficient free space to copy these files into"
        print "                      $prepareDir."
        print

        listOfMiscTsmServerFiles=""
        listOfFilesToCheck="$pathToTsmServerFiles/$fileA $fileB $fileC"
        for eachFileToCheck in $listOfFilesToCheck
        do
          if [ "$eachFileToCheck" = '?' ]
          then
            :  # Proceed normally
          else
            listOfMiscTsmServerFiles="$listOfMiscTsmServerFiles $eachFileToCheck "
          fi
        done

        print "TSMMISCFILESSAVED $listOfMiscTsmServerFiles" >> $prepareDir/$criticalSysInfo
        print >> $prepareDir/$criticalSysInfo
      else
        writeErr "NSM-95132-E: ERROR($scriptName) - These files are too large to be copied into $prepareDir."
        writeErr "                                  You must manually FTP them to another machine or save them off."
        writeErr
        writeErr "                                  You may either:"
        writeErr
        writeErr "                                  <1> Exit and clean up $prepareDir before retrying this"
        writeErr "                                      script, or"
        writeErr
        writeErr "                                  <2> Open a new window to move these files off manually, and then"
        writeErr "                                      continuing."
        writeErr
        print "TSMMISCFILESSAVED Manual" >> $prepareDir/$criticalSysInfo
        print >> $prepareDir/$criticalSysInfo
        cOrEAndEnterToContinue
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END checkMiscTsmServerFileSizes

# ---------------------------------------------------------------------------
# Formats a 1.44mb diskette
# ---------------------------------------------------------------------------

function formatFloppy {

  subprocessName='formatFloppy'
  if [ "$skipFormatFloppy" = 'no' ]
  then
    format 2>/dev/null
    formatRC=$?
    if [ "$formatRC" = '0' ]
    then
      print
      print "INFO: ($scriptName) - Diskette successfully formatted."
      print
    else
      writeErr
      writeErr "NSM-95134-E: ERROR($scriptName) - Unable to format diskette!"
      writeErr
      programRC=1
      return $programRC
    fi
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
    programRC=0
  fi

}  # END formatFloppy

# ---------------------------------------------------------------------------
# Save Tivoli Disaster Recovery Manager (DRM) files (if licensed)
# ---------------------------------------------------------------------------

function saveDrmFiles {

  subprocessName='saveDrmFiles'
  if [ "$skipSaveDrmFiles" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    isDrmLicensed=`$tsmInvoc -commadelimit select "'keyID'",DRM_LIC from licenses | grep 'keyID' | grep -v 'select' | cut -f2- -d','`
    if [ "$isDrmLicensed" = 'Yes' ]
    then
      print "INFO: ($scriptName) - Tivoli Disaster Recovery Manager (DRM) is licensed on"
      print "                      this server."
      print
      mkdir $prepareDir/$relativePathToDrm1Files $prepareDir/$relativePathToDrm2Files

      # ---------------------------------------------------------------------
      # Copy the DRM Recovery Instructions files

      print "INFO: ($scriptName) - Copying recovery instructions files ..."
      print

      recoveryInstructionsPrefix=`$tsmInvoc -commadelimit select "'keyID'",INSTRPREFIX from drmstatus | grep 'keyID' | grep -v 'select' | cut -f2- -d','`

      if [ "$recoveryInstructionsPrefix" = '?' ]
      then
        recoveryInstructionsPrefix=''  # TSM Server uses '?' as empty value for SQL query
      fi

      if [ "$recoveryInstructionsPrefix" = '' ]
      then
        if [ -f $pathToTsmServerFiles/RECOVERY.INSTRUCTIONS.* ]
        then
          instructionsFiles=`ls $pathToTsmServerFiles | grep RECOVERY.INSTRUCTIONS.*`
          for eachFile in $instructionsFiles
          do
            print "                      $pathToTsmServerFiles/$eachFile -- \c"
            cp $pathToTsmServerFiles/$eachFile $prepareDir/$relativePathToDrm1Files 2>/dev/null
            cpRC=$?
            if [ "$cpRC" = '0' ]
            then
              print "successful"
              drm1FilesFound="$drm1FilesFound $eachFile "
            else
              print "!! FAILED !!"
              listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$eachFile "
            fi
          done
          print
          print "DRMINSTRPR $pathToTsmServerFiles $drm1FilesFound" >> $prepareDir/$criticalSysInfo
          print >> $prepareDir/$criticalSysInfo
        else
          print "INFO: ($scriptName) - No recovery instructions files found."
          print
        fi
      else
        print "INFO: ($scriptName) - A prefix \"$recoveryInstructionsPrefix\" exists."
        print
        files="RECOVERY.INSTRUCTIONS*"
        lengthOfPrefix=${#recoveryInstructionsPrefix}
        firstChar=$(print "$recoveryInstructionsPrefix" | cut -c 1-1)
        lastChar=$(print "$recoveryInstructionsPrefix" | cut -c $lengthOfPrefix-$lengthOfPrefix)
        if [ "$lastChar" = '/' ]
        then
          print "INFO: ($scriptName) - The prefix is a directory."
          print
          if [ "$firstChar" = '/' ]
          then
            # ---------------------------------------------------------------
            # e.g., set drminstrpr /tmp/

            if [ -f "$recoveryInstructionsPrefix"RECOVERY.INSTRUCTIONS* ]
            then
              instructionsFiles=`ls $recoveryInstructionsPrefix | grep RECOVERY.INSTRUCTIONS*`
              for eachFile in $instructionsFiles
              do
                print "                      $recoveryInstructionsPrefix$eachFile -- \c"
                cp $recoveryInstructionsPrefix$eachFile $prepareDir/$relativePathToDrm1Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm1FilesFound="$drm1FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $recoveryInstructionsPrefix$eachFile "
                fi
              done
              print
              print "DRMINSTRPR $recoveryInstructionsPrefix $drm1FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery instructions files found."
              print
            fi
          else
            # ---------------------------------------------------------------
            # e.g., set drminstrpr tmp/

            if [ -f "$pathToTsmServerFiles/$recoveryInstructionsPrefix"RECOVERY.INSTRUCTIONS* ]
            then
              instructionsFiles=`ls $pathToTsmServerFiles/$recoveryInstructionsPrefix | grep RECOVERY.INSTRUCTIONS*`
              for eachFile in $instructionsFiles
              do
                print "                      $pathToTsmServerFiles/$recoveryInstructionsPrefix$eachFile -- \c"
                cp $pathToTsmServerFiles/$recoveryInstructionsPrefix$eachFile $prepareDir/$relativePathToDrm1Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm1FilesFound="$drm1FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$recoveryInstructionsPrefix$eachFile "
                fi
              done
              print
              print "DRMINSTRPR $pathToTsmServerFiles/$recoveryInstructionsPrefix $drm1FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery instructions files found."
              print
            fi
          fi
        else
          print "INFO: ($scriptName) - The prefix \"$recoveryInstructionsPrefix.$files\" is a string."
          print
          if [ "$firstChar" = '/' ]
          then
            # ---------------------------------------------------------------
            # e.g., set drminstrpr /tmp

            print "INFO: ($scriptName) - The string has a directory associated with it."
            print
            if [ -f $recoveryInstructionsPrefix.RECOVERY.INSTRUCTIONS* ]
            then
              recoveryInstructionsPrefix=`print $recoveryInstructionsPrefix | cut -b2-`
              instructionsFiles=`ls / | grep $recoveryInstructionsPrefix.RECOVERY.INSTRUCTIONS*`
              for eachFile in $instructionsFiles
              do
                print "                      /$eachFile -- \c"
                cp /$eachFile $prepareDir/$relativePathToDrm1Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm1FilesFound="$drm1FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles /$eachFile "
                fi
              done
              print
              print "DRMINSTRPR / $drm1FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery instructions files found."
              print
            fi
          else
            # ---------------------------------------------------------------
            # e.g., set drminstrpr tmp

            print "INFO: ($scriptName) - The string has no directory associated with it."
            print
            if [ -f $pathToTsmServerFiles/$recoveryInstructionsPrefix.RECOVERY.INSTRUCTIONS* ]
            then
              instructionsFiles=`ls $pathToTsmServerFiles | grep $recoveryInstructionsPrefix.RECOVERY.INSTRUCTIONS*`
              for eachFile in $instructionsFiles
              do
                print "                      $pathToTsmServerFiles/$eachFile -- \c"
                cp $pathToTsmServerFiles/$eachFile $prepareDir/$relativePathToDrm1Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm1FilesFound="$drm1FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$eachFile "
                fi
              done
              print
              print "DRMINSTRPR $pathToTsmServerFiles $drm1FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery instructions files found."
              print
            fi
          fi
        fi
      fi

      # ---------------------------------------------------------------------
      # Copy the DRM Recovery Plan files

      print "INFO: ($scriptName) - Copying recovery plan files ..."
      print

      recoveryPlanPrefix=`$tsmInvoc -commadelimit select "'keyID'",PLANPREFIX from drmstatus | grep 'keyID' | grep -v 'select' | cut -f2- -d','`

      if [ "$recoveryPlanPrefix" = '' ]
      then
        print "INFO: ($scriptName) - Recovery Plan Prefix not defined."
        print
      else
        print "INFO: ($scriptName) - A prefix \"$recoveryPlanPrefix\" exists."
        print
        files="*"
        lengthOfPrefix=${#recoveryPlanPrefix}
        firstChar=$(print "$recoveryPlanPrefix" | cut -c 1-1)
        lastChar=$(print "$recoveryPlanPrefix" | cut -c $lengthOfPrefix-$lengthOfPrefix)

        if [ "$lastChar" = '/' ]
        then
          print "INFO: ($scriptName) - The prefix is a directory."
          print
          if [ "$firstChar" = '/' ]
          then
            # ---------------------------------------------------------------
            # e.g., set drmplanpr /tmp/

            if [ -d "$recoveryPlanPrefix" ]
            then
              planFiles=`ls $recoveryPlanPrefix`
            else
              planFiles=''
            fi

            if [ "$planFiles" = '' ]
            then
              print "INFO: ($scriptName) - No recovery plan files found."
              print
            else
              for eachFile in $planFiles
              do
                print "                      $recoveryPlanPrefix$eachFile -- \c"
                cp $recoveryPlanPrefix$eachFile $prepareDir/$relativePathToDrm2Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm2FilesFound="$drm2FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $recoveryPlanPrefix$eachFile "
                fi
              done
              print
              print "DRMPLANPR $recoveryPlanPrefix $drm2FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            fi
          else
            # ---------------------------------------------------------------
            # e.g., set drmplanpr tmp/

            if [ -d "$pathToTsmServerFiles/$recoveryPlanPrefix" ]
            then
              planFiles=`ls $pathToTsmServerFiles/$recoveryPlanPrefix`
            else
              planFiles=''
            fi

            if [ "$planFiles" = '' ]
            then
              print "INFO: ($scriptName) - No recovery plan files found."
              print
            else
              for eachFile in $planFiles
              do
                print "                      $pathToTsmServerFiles/$recoveryPlanPrefix$eachFile -- \c"
                cp $pathToTsmServerFiles/$recoveryPlanPrefix$eachFile $prepareDir/$relativePathToDrm2Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm2FilesFound="$drm2FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$recoveryPlanPrefix$eachFile "
                fi
              done
              print
              print "DRMPLANPR $pathToTsmServerFiles/$recoveryPlanPrefix $drm2FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            fi
          fi
        else
          print "INFO: ($scriptName) - The prefix \"$recoveryPlanPrefix.$files\" is a string."
          print
          if [ "$firstChar" = '/' ]
          then
            # ---------------------------------------------------------------
            # e.g., set drmplanpr /tmp

            print "INFO: ($scriptName) - The string has a directory associated with it."
            print
            if [ -f $recoveryPlanPrefix.$files ]
            then
              recoveryPlanPrefix=`print $recoveryPlanPrefix | cut -b2-`
              planFiles=`ls / | grep $recoveryPlanPrefix.$files`
              for eachFile in $planFiles
              do
                print "                      /$eachFile -- \c"
                cp /$eachFile $prepareDir/$relativePathToDrm2Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm2FilesFound="$drm2FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles /$eachFile "
                fi
              done
              print
              print "DRMPLANPR / $drm2FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery plan files found."
              print
            fi
          else
            # ---------------------------------------------------------------
            # e.g., set drmplanpr tmp

            print "INFO: ($scriptName) - The string has no directory associated with it."
            print
            if [ -f $pathToTsmServerFiles/$recoveryPlanPrefix.$files ]
            then
              planFiles=`ls $pathToTsmServerFiles | grep $recoveryPlanPrefix.$files`
              for eachFile in $planFiles
              do
                print "                      $pathToTsmServerFiles/$eachFile -- \c"
                cp $pathToTsmServerFiles/$eachFile $prepareDir/$relativePathToDrm2Files 2>/dev/null
                cpRC=$?
                if [ "$cpRC" = '0' ]
                then
                  print "successful"
                  drm2FilesFound="$drm2FilesFound $eachFile "
                else
                  print "!! FAILED !!"
                  listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$eachFile "
                fi
              done
              print
              print "DRMPLANPR $pathToTsmServerFiles $drm2FilesFound" >> $prepareDir/$criticalSysInfo
              print >> $prepareDir/$criticalSysInfo
            else
              print "INFO: ($scriptName) - No recovery plan files found."
              print
            fi
          fi
        fi
      fi
    else
      print "INFO: ($scriptName) - Tivoli Disaster Recovery Manager (DRM) is not licensed on"
      print "                      this server.  No files to copy."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END saveDrmFiles

# ---------------------------------------------------------------------------
# Restore Tivoli Disaster Recovery Manager (DRM) files (if saved)
# ---------------------------------------------------------------------------

function restoreDrmFiles {

  subprocessName='restoreDrmFiles'
  if [ "$skipRestoreDrmFiles" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    drmFilesExist=`cat $restoreDir/$criticalSysInfo | grep DRM`
    if [ "$drmFilesExist" = '' ]
    then
      print "INFO: ($scriptName) - No Tivoli Disaster Recovery Manager (DRM) files to"
      print "                      restore."
      print
    else
      drminstrprFilesExist=`cat $restoreDir/$criticalSysInfo | grep DRMINSTRPR`
      if [ "$drminstrprFilesExist" = '' ]
      then
        :  # Proceed normally
      else
        targetDir=`print $drminstrprFilesExist | awk '{print $2}'`
        if [ -d $targetDir ]
        then
          :  # Proceed normally
        else
          mkdir -p $targetDir 2>/dev/null
        fi

        print "INFO: ($scriptName) - Copying recovery instructions files to"
        print "                      $targetDir."
        print

        filesToCopy=`print $drminstrprFilesExist | cut -f3- -d' '`
        for eachFile in $filesToCopy
        do
          print "                      $targetDir/$eachFile -- \c"
          cp $restoreDir/$relativePathToDrm1Files/$eachFile $targetDir 2>/dev/null
          cpRC=$?
          if [ "$cpRC" = '0' ]
          then
            print "successful"
            drm1FilesFound="$drm1FilesFound $eachFile "
          else
            print "!! FAILED !!"
            listOfFailedFiles="$listOfFailedFiles $restoreDir/$relativePathToDrm1Files/$eachFile "
          fi
        done
        print
      fi

      drmplanprFilesExist=`cat $restoreDir/$criticalSysInfo | grep DRMPLANPR`
      if [ "$drmplanprFilesExist" = '' ]
      then
        :  # Proceed normally
      else
        targetDir=`print $drmplanprFilesExist | awk '{print $2}'`
        if [ -d $targetDir ]
        then
          :  # Proceed normally
        else
          mkdir -p $targetDir 2>/dev/null
        fi

        print "INFO: ($scriptName) - Copying recovery plan files to"
        print "                      $targetDir."
        print

        filesToCopy=`print $drmplanprFilesExist | cut -f3- -d' '`
        for eachFile in $filesToCopy
        do
          print "                      $targetDir/$eachFile -- \c"
          cp $restoreDir/$relativePathToDrm2Files/$eachFile $targetDir 2>/dev/null
          cpRC=$?
          if [ "$cpRC" = '0' ]
          then
            print "successful"
            drm2FilesFound="$drm2FilesFound $eachFile "
          else
            print "!! FAILED !!"
            listOfFailedFiles="$listOfFailedFiles $restoreDir/$relativePathToDrm2Files/$eachFile "
          fi
        done
        print
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END restoreDrmFiles

# ---------------------------------------------------------------------------
# Save all the critical files to diskette
# ---------------------------------------------------------------------------

function saveFilesToFloppy {

  subprocessName='saveFilesToFloppy'
  if [ "$skipSaveFilesToFloppy" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Please insert the provided 1.44mb disk into the diskette"
    print "                      drive.  This diskette will be formatted, and critical"
    print "                      system files collected in Phase 1 will be copied to it"
    print "                      for use in Phase 2."
    print

    function handleIt {
      formatFloppy
      formatFloppyRC=$?
      if [ "$formatFloppyRC" = '0' ]
      then
        :  # Proceed normally
      else
        response=''
        until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
        do
          read response?"                                  Do you wish to try again [y,n]? "
          print
        done

        if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
        then
          handleIt
        elif [ "$response" = 'n' ] || [ "$response" = 'N' ]
        then
          writeErr "NSM-95135-E: ERROR($scriptName) - Unable to save critical system files to diskette!"
          writeErr
          eAndEnterToExit
        fi
      fi
    }  # END handleIt

    handleIt

    # -----------------------------------------------------------------------
    # Copy the TSM client configuration files over to $prepareDir first

    print "INFO: ($scriptName) - Copying TSM client config files to $prepareDir:"
    print
    for eachFile in $tsmClientFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToTsmClientFiles/$eachFile" ]
      then
        cp $pathToTsmClientFiles/$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          tsmClientFilesFound="$tsmClientFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToTsmClientFiles/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the TSM server configuration files over to $prepareDir first

    print "INFO: ($scriptName) - Copying TSM server config files to $prepareDir:"
    print
    for eachFile in $tsmServerFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToTsmServerFiles/$eachFile" ]
      then
        cp $pathToTsmServerFiles/$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          tsmServerFilesFound="$tsmServerFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToTsmServerFiles/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the /etc configuration files over to $prepareDir first

    print "INFO: ($scriptName) - Copying $pathToEtcFiles config files to $prepareDir:"
    print
    for eachFile in $etcFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToEtcFiles/$eachFile" ]
      then
        cp $pathToEtcFiles/$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          etcFilesFound="$etcFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToEtcFiles/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy additional /etc configuration files over to $prepareDir first, but
    # these will not be restored later

    print "INFO: ($scriptName) - Copying additional $pathToEtcFiles config files to $prepareDir:"
    print
    for eachFile in $etcFiles2
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToEtcFiles/$eachFile" ]
      then
        cp $pathToEtcFiles/$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          etcFiles2Found="$etcFiles2Found $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToEtcFiles/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Make a copy of the .profile in root and copy it over to $prepareDir
    # first

    cp "$pathToRootFiles".profile $pathToRootFiles$rootFiles 2>/dev/null
    print "INFO: ($scriptName) - Copying root config files to $prepareDir:"
    print
    for eachFile in $rootFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToRootFiles$eachFile" ]
      then
        cp $pathToRootFiles$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          rootFilesFound="$rootFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToRootFiles$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the sysInfo configuration files over to $prepareDir first

    print "INFO: ($scriptName) - Copying $pathToSysInfoFiles config files to"
    print "                      $prepareDir:"
    print
    for eachFile in $sysInfoFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$pathToSysInfoFiles/$eachFile" ]
      then
        cp $pathToSysInfoFiles/$eachFile $prepareDir 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          sysInfoFilesFound="$sysInfoFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $pathToSysInfoFiles/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy miscellaneous TSM server info files over to $prepareDir

    if [ "$miscTsmFiles" = 'no' ]
    then
      :  # Proceed normally
    else
      print "INFO: ($scriptName) - Copying misc TSM info files to $prepareDir:"
      print
      for eachFile in $listOfMiscTsmServerFiles
      do
        print "                      $eachFile -- \c"
        if [ -f "$eachFile" ]
        then
          cp $eachFile $prepareDir 2>/dev/null
          cpRC=$?
          if [ "$cpRC" = '0' ]
          then
            print "successful"
            listOfMiscTsmServerFilesFound="$listOfMiscTsmServerFilesFound $eachFile "
          else
            print "!! FAILED !!"
            listOfFailedFiles="$listOfFailedFiles $eachFile "
          fi
        else
          print "** MISSING **"
        fi
      done
      print
    fi

    # -----------------------------------------------------------------------
    # Copy Tivoli Disaster Recovery Manager (DRM) files over to $prepareDir
    # first

    saveDrmFiles

    if [ -d "$prepareDir/$relativePathToDrm1Files" ]
    then
      files=`ls $prepareDir/$relativePathToDrm1Files`
      for eachFile in $files
      do
        drm1="$drm1 $relativePathToDrm1Files/$eachFile "
      done
    else
      drm1=""
    fi

    if [ -d "$prepareDir/$relativePathToDrm2Files" ]
    then
      files=`ls $prepareDir/$relativePathToDrm2Files`
      for eachFile in $files
      do
        drm2="$drm2 $relativePathToDrm2Files/$eachFile "
      done
    else
      drm2=""
    fi

    # -----------------------------------------------------------------------
    # Copy SSA disk information files over to $prepareDir first

    print "INFO: ($scriptName) - Copying $pathToSsaDiskFiles info files to"
    print "                      $prepareDir:"
    print
    mkdir $prepareDir/$relativePathToSsaDiskFiles
    for eachFile in $ssaDiskFiles
    do
      print "                      $eachFile -- \c"
      cp $pathToSsaDiskFiles/$eachFile $prepareDir/$relativePathToSsaDiskFiles 2>/dev/null
      cpRC=$?
      if [ "$cpRC" = '0' ]
      then
        print "successful"
        ssaDiskFilesFound="$ssaDiskFilesFound $relativePathToSsaDiskFiles/$eachFile "
      else
        print "!! FAILED !!"
        listOfFailedFiles="$listOfFailedFiles $prepareDir/$relativePathToSsaDiskFiles/$eachFile "
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy lsSSlevel (WebShlLvl.*) files over to $prepareDir first

    print "INFO: ($scriptName) - Copying lsSSlevel info files to $prepareDir:"
    print
    mkdir $prepareDir/$relativePathToSsLevelFiles

    for eachFile in $ssLevelFiles
    do
      print "                      $eachFile -- \c"
      cp $pathToSsLevelFiles/$eachFile $prepareDir/$relativePathToSsLevelFiles 2>/dev/null
      cpRC=$?
      if [ "$cpRC" = '0' ]
      then
        print "successful"
        ssLevelFilesFound="$ssLevelFilesFound $relativePathToSsLevelFiles/$eachFile "
      else
        print "!! FAILED !!"
        listOfFailedFiles="$listOfFailedFiles $prepareDir/$relativePathToSsLevelFiles/$eachFile "
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Tar, Compress, and then Backup the archive to floppy

    print "INFO: ($scriptName) - Copying archive to diskette ..."
    print
    problemHappened='no'

    theCommand="originalDir=`pwd`"
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="cd $prepareDir"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="tar -cvf $archivedFiles $tsmClientFilesFound $tsmServerFilesFound $etcFilesFound $etcFiles2Found $rootFilesFound $sysInfoFilesFound $listOfMiscTsmServerFilesFound $drm1 $drm2 $ssaDiskFilesFound $ssLevelFilesFound $genericSysInfo $criticalSysInfo $getTapeInfoBeforeFile $captureIPenvironmentFile"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="compress $archivedFiles"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    sizeOfArchivedFiles=`ls -l $archivedFiles.Z | awk '{print $5}'`
    if [ "$sizeOfArchivedFiles" -gt "$maxSizeOfDiskette" ]
    then
      moreThanOneDisketteNeeded='yes'
      howManyExtraDiskettesNeeded=$((sizeOfArchivedFiles / maxSizeOfDiskette)) 
      print "INFO: ($scriptName) - $archivedFiles.Z cannot be archived to one diskette."
      if [ "$howManyExtraDiskettesNeeded" = '1' ]
      then
        print "                      Please format $howManyExtraDiskettesNeeded more 1.44mb diskette before continuing."
        print
      else
        print "                      Please format $howManyExtraDiskettesNeeded more 1.44mb diskettes before continuing."
        print
      fi
      cOrEAndEnterToContinue
    fi

    function archiveToDiskette {
      theCommand="tar -cvf /dev/fd0 $archivedFiles.Z"
      print "                      Issuing command: \"$theCommand\""
      print
      eval $theCommand > /dev/null
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ] && [ "$problemHappened" = 'no' ]
      then
        print
        print "INFO: ($scriptName) - Archive successfully copied to diskette."
        print
      else
        writeErr
        writeErr "NSM-95136-E: ERROR($scriptName) - There was a problem copying the archive to diskette!"
        writeErr "                                  ($prepareDir/$archivedFiles.Z)."
        writeErr
        programRC=1
        return $programRC
      fi
    }  # END archiveToDiskette

    function handleIt {
      archiveToDiskette
      archiveToDisketteRC=$?
      if [ "$archiveToDisketteRC" = '0' ]
      then
        :  # Proceed normally
      else
        response=''
        until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
        do
          read response?"                                  Do you wish to try again [y,n]? "
          print
        done

        if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
        then
          handleIt
        elif [ "$response" = 'n' ] || [ "$response" = 'N' ]
        then
          writeErr "NSM-95135-E: ERROR($scriptName) - Unable to save critical system files to diskette!"
          writeErr
          eAndEnterToExit
        fi
      fi
    }  # END handleIt

    handleIt

    # -----------------------------------------------------------------------
    # Check the integrity of the archive by restoring from floppy and
    # uncompressing and untarring

    if [ "$moreThanOneDisketteNeeded" = 'yes' ]
    then
      print "INFO: ($scriptName) - Please insert diskette #1 back into floppy drive."
      print
      cAndEnterToContinue
    fi

    print "INFO: ($scriptName) - Checking archive integrity ..."
    print
    problemHappened='no'

    theCommand="mkdir /tmp/$archivedFiles.$timeStamp"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="cd /tmp/$archivedFiles.$timeStamp"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="tar -xvf /dev/fd0"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="uncompress $archivedFiles.Z"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" != '0' ]
    then
      problemHappened='yes'
    fi

    theCommand="tar -xvf $archivedFiles"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ] && [ "$problemHappened" = 'no' ]
    then
      print "INFO: ($scriptName) - Archive integrity successfully verified."
      print
    else
      writeErr "NSM-95137-E: ERROR($scriptName) - There was a problem verifying archive integrity!"
      writeErr "                                  Please take corrective action."
      eAndEnterToExit
    fi

    theCommand="cd $originalDir"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?

    theCommand="rm -r /tmp/$archivedFiles.$timeStamp"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?

    print "INFO: ($scriptName) - Please remove the diskette and put it in a safe location,"
    print "                      however make sure it is available for Phase 2."
    print
    cAndEnterToContinue

    # -----------------------------------------------------------------------
    # Copy the reclog to /adsmlog before exporting adsmvg

    cp -rph /tmp/$scriptName $prepareDir/$scriptName.phase1 > /dev/null

    subprocessName='saveFilesToFloppy'
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END saveFilesToFloppy

# ---------------------------------------------------------------------------
# Run prebackup exportit
# ---------------------------------------------------------------------------

function prebackupExportit {

  subprocessName='prebackupExportit'
  if [ "$skipPrebackupExportit" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeEntering

    theCommand="prebackup exportit"
    print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
    print
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print
      print "INFO: ($scriptName) - Prebackup completed successfully."
      print
    else
      writeErr
      writeErr "INFO: ($scriptName) - Prebackup did not complete successfully!  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeSkipping
  fi

}  # END prebackupExportit

# ---------------------------------------------------------------------------
# Closes out Phase 1 and performs a shutdown -Fr if desired
# ---------------------------------------------------------------------------

function endPhase1 {

  subprocessName='endPhase1'
  if [ "$skipEndPhase1" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Removing automatic start of the TSM server ..."
    print

    autoTsmStatus=`lsitab -a | grep autosrvr`
    if [ "$autoTsmStatus" = '' ]
    then
      print "INFO: ($scriptName) - No entry was found in the initialization table."
      print
    else
      rmitab "autosrvr" 2>/dev/null
      removeautoRC=$?
      if [ "$removeautoRC" = '0' ]
      then
        print "INFO: ($scriptName) - Successfully removed entry from initialization table."
        print
      else
        print "NSM-95138-W: WARNING($scriptName) - Could not remove entry from initialization table."
        print
        cOrEAndEnterToContinue
      fi
    fi

    function displayHeader {
      print
      print "+-----------------------------------------------------------------------------+"
      print "|                                                                             |"
      print "| Phase 1 has completed successfully.  Please insert the provided mksysb tape |"
      print "| into the 4mm drive and follow the directions in the Installation            |"
      print "| Instructions (II) to change the boot order in SMS.  To continue with the    |"
      print "| upgrade, the system must be restarted now.                                  |"
      print "|                                                                             |"
      print "+-----------------------------------------------------------------------------+"
      print
      print
    }  # END displayHeader

    function displayMenu {
      print "[1] Restart the system with the \"shutdown -Fr\" command."
      print
      print "[2] No, I want to exit this script now and restart later."
      print
    }  # END displayMenu

    displayHeader
    response=0
    until [ "$response" = '1' ] || [ "$response" = '2' ]
    do
      displayMenu
      read response?"Please make your selection [1,2]: "
      if [ "$response" = '1' ] || [ "$response" = '2' ]
      then
        : # Pass through
      else
        displayHeader
        print "-> $response <- is an invalid response!  Please try again."
      fi
      print
      print
    done

    case $response in
      (1)
        print "INFO: ($scriptName) - User selected Option [1] -- Restart system now."
        print

        if [[ -f $adsmlogDir/$storagePoolMigration ]]
        then
          rm $adsmlogDir/$storagePoolMigration
        fi

        shutdown -Fr
        ;;
      (2)
        print "INFO: ($scriptName) - User selected Option [2] -- Restart system later."
        print
        programRC=0
        return $programRC
        ;;
    esac

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END endPhase1

# ---------------------------------------------------------------------------
# Run fixSSA
# ---------------------------------------------------------------------------

function callFixSsa {

  subprocessName='callFixSsa'
  if [ "$skipCallFixSsa" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeEntering

    isThisRaid5=`ssaraid -Iz -l ssa0 | grep array`
    if [ "$isThisRaid5" = '' ]
    then
      theCommand="fixSSA"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ]
      then
        print
        print "INFO: ($scriptName) - fixSSA completed successfully."
        print
      else
        writeErr
        writeErr "NSM-95139-E: ERROR($scriptName) - fixSSA did not complete successfully!  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    else
      print "INFO: ($scriptName) - This is a RAID 5 system.  The fixSSA script will not be"
      print "                      executed."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeSkipping
  fi

}  # END callFixSsa

# ---------------------------------------------------------------------------
# Restores all the critical files from diskette
# ---------------------------------------------------------------------------

function restoreFilesFromFloppy {

  subprocessName='restoreFilesFromFloppy'
  if [ "$skipRestoreFilesFromFloppy" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Please insert the disk created in Phase 1 into the"
    print "                      diskette drive.  Saved files will be restored to:"
    print
    print "                      $restoreDir"
    print
    cAndEnterToContinue

    print "INFO: ($scriptName) - Copying archive from diskette ..."
    print

    theCommand="mkdir -p $restoreDir"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      :  # Proceed normally
    else
      writeErr "NSM-95140-E: ERROR($scriptName) - Unable to create $restoreDir!"
      writeErr "                                  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    theCommand="cd $restoreDir"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      :  # Proceed normally
    else
      writeErr "NSM-95141-E: ERROR($scriptName) - Unable to cd to $restoreDir!"
      writeErr "                                  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    function retrieveFromDiskette {
      theCommand="tar -xvf /dev/fd0"
      print "                      Issuing command: \"$theCommand\""
      print
      eval $theCommand > /dev/null
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr
        writeErr "NSM-95142-E: ERROR($scriptName) - Unable to restore archive from diskette!"
        writeErr
        programRC=1
        return $programRC
      fi
    }  # END retrieveFromDiskette

    function handleIt {
      retrieveFromDiskette
      retrieveFromDisketteRC=$?
      if [ "$retrieveFromDisketteRC" = '0' ]
      then
        :  # Proceed normally
      else
        response=''
        until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
        do
          read response?"                                  Do you wish to try again [y,n]? "
          print
        done

        if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
        then
          handleIt
        elif [ "$response" = 'n' ] || [ "$response" = 'N' ]
        then
          writeErr "NSM-95160-E: ERROR($scriptName) - Unable to restore critical system files from diskette!"
          writeErr
          eAndEnterToExit
        fi
      fi
    }  # END handleIt

    handleIt

    theCommand="uncompress $archivedFiles.Z"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      :  # Proceed normally
    else
      writeErr "NSM-95143-E: ERROR($scriptName) - Unable to uncompress $archivedFiles.Z!"
      writeErr "                                  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    theCommand="tar -xvf $archivedFiles"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      :  # Proceed normally
    else
      writeErr "NSM-95144-E: ERROR($scriptName) - Unable to extract files from $archivedFiles!"
      writeErr "                                  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    theCommand="rm $archivedFiles"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null

    print "INFO: ($scriptName) - Archive successfully restored from diskette."
    print

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END restoreFilesFromFloppy

# ---------------------------------------------------------------------------
# Updates the 3466.model file
# ---------------------------------------------------------------------------

function updateModel {

  subprocessName='updateModel'
  if [ "$skipUpdateModel" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    osLevel=`oslevel`
    if [ "$osLevel" = "$targetOsLevel" ]
    then
      print "INFO: ($scriptName) - Verified AIX OS Level at $osLevel."
      print
    else
      writeErr "NSM-95145-E: ERROR($scriptName) - AIX $osLevel was detected, but $targetOsLevel is expected!"
      writeErr "                                  Please contact IBM support."
      eAndEnterToExit
    fi

    dateStamp=`date`
    existingModelInfo=`cat $restoreDir/3466.model | awk '{print $1" "$2" "$3" "$4" "$5}'`
    newModelInfo=`print $existingModelInfo updated to $thisEcLevel on $dateStamp`

    theCommand="print $newModelInfo > $pathToSysInfoFiles/3466.model"
    eval $theCommand > /dev/null
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print "INFO: ($scriptName) - 3466.model successfully updated."
      print
    else
      writeErr "NSM-95146-E: ERROR($scriptName) - Unable to update 3466.model!  Please call IBM support."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END updateModel

# ---------------------------------------------------------------------------
# Check the system date and time zone
# ---------------------------------------------------------------------------

function checkSystemDateTz {

  subprocessName='checkSystemDateTz'
  if [ "$skipCheckSystemDateTz" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    oldTimeZone=$(cat $restoreDir/$criticalSysInfo | grep TIMEZONE | awk '{print $2}')
    newTimeZone=$(date | awk '{print $5}')

    if [ "$oldTimeZone" = "$newTimeZone" ]
    then
      :  # Proceed normally
    else
      writeErr "NSM-95147-E: ERROR($scriptName) - The current time zone does not match the original time zone!"
      writeErr
      writeErr "                                   Current: $newTimeZone"
      writeErr "                                  Original: $oldTimeZone"
      writeErr
      writeErr "                                  Please take corrective action via \"smitty chtz_date\" and then"
      writeErr "                                  restart the system before restarting this script."

      writeErr
      eAndEnterToExit
    fi

    currentDate=`date "+(%A) %B %d, %Y"`
    currentTime=`date "+%I:%M %p"`

    print "INFO: ($scriptName) - The current date and time are:"
    print
    print "                      $currentDate  -*-  $currentTime"
    print

    response=''
    until [ "$response" = 'y' ] || [ "$response" = 'Y' ] || [ "$response" = 'n' ] || [ "$response" = 'N' ]
    do
      read response?"                      Is this correct [y,n]? "
      print
      print
    done

    if [ "$response" = 'y' ] || [ "$response" = 'Y' ]
    then
      print "INFO: ($scriptName) - User selected [$response] -- Yes, date and time are correct."
      print
    else
      print "INFO: ($scriptName) - User selected [$response] -- No, date and time are not correct."
      print
      writeErr "INFO: ($scriptName) - Please take corrective action via \"smitty date\" before restarting this"
      writeErr "                      script."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END checkSystemDateTz

# ---------------------------------------------------------------------------
# Run postbackup importit
# ---------------------------------------------------------------------------

function postbackupImportit {

  subprocessName='postbackupImportit'
  if [ "$skipPostbackupImportit" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    theCommand="postbackup importit"
    print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
    print
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print
      print "INFO: ($scriptName) - Postbackup completed successfully."
      print
    else
      writeErr
      writeErr "INFO: ($scriptName) - Postbackup did not complete successfully!  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END postbackupImportit

# ---------------------------------------------------------------------------
# Check general SSA health and VG quorum
# ---------------------------------------------------------------------------

function checkSsa {

  subprocessName='checkSsa'
  if [ "$skipCheckSsa" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    theCommand="vdisk -c -v"
    print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      :  # Proceed normally
    else
      writeErr
      writeErr "INFO: ($scriptName) - A problem was detected verifying SSA disks!"
      writeErr "                      Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    theCommand="chvg -a'y' -Q'y' -x'n' adsmvg"
    print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
    print
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print "INFO: ($scriptName) - Quorum and vgonline updated for adsmvg."
      print
    else
      writeErr
      writeErr "INFO: ($scriptName) - Unable to update quorum and vgonline for adsmvg!"
      writeErr "                      Please take corrective action."
      writeErr
      eAndEnterToExit
    fi

    adsmvg2Exists=$(lsdev -Cc disk | grep -c SSA)
    if [ "$adsmvg2Exists" -gt '32' ]
    then
      theCommand="chvg -a'y' -Q'y' -x'n' adsmvg2"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ]
      then
        print "INFO: ($scriptName) - Quorum and vgonline updated for adsmvg2."
        print
      else
        writeErr
        writeErr "INFO: ($scriptName) - Unable to update quorum and vgonline for adsmvg2!"
        writeErr "                      Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END checkSsa

# ---------------------------------------------------------------------------
# Removes files left over from the build process
# ---------------------------------------------------------------------------

function removeExtraFiles {

  subprocessName='removeExtraFiles'
  if [ "$skipRemoveExtraFiles" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Removing extra files ..."
    print

    for eachFile in $extraFiles
    do
      if [ -f $eachFile ]
      then
        theCommand="rm $eachFile"
        print "                      Issuing command: \"$theCommand\""
        print
        eval $theCommand > /dev/null
        fileFound='yes'
      fi
    done
    if [ "$fileFound" = 'yes' ]
    then
      :  # Proceed normally
    else
      print "INFO: ($scriptName) - No extra files found."
      print
    fi

    print "INFO: ($scriptName) - Removing extra directories ..."
    print

    for eachDir in $extraDirs
    do
      if [ -d $eachDir ]
      then
        theCommand="rm -r $eachDir"
        print "                      Issuing command: \"$theCommand\""
        print
        eval $theCommand > /dev/null
        dirFound='yes'
      fi
    done
    if [ "$dirFound" = 'yes' ]
    then
      :  # Proceed normally
    else
      print "INFO: ($scriptName) - No extra directories found."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END removeExtraFiles

# ---------------------------------------------------------------------------
# Deletes rmt0 after the mksysb restore in case it's the wrong type
# ---------------------------------------------------------------------------

function removeRmt0 {

  subprocessName='removeRmt0'
  if [ "$skipRemoveRmt0" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    doesRmt0Exist=`lscfg -vl rmt0`
    if [ "$doesRmt0Exist" = '' ]
    then
      print "INFO: ($scriptName) - rmt0 is not defined."
      print
    else
      theCommand="rmdev -l rmt0 -d"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr
        writeErr "NSM-95148-W: WARNING($scriptName) - Unable to delete rmt0.  Corrective action may be necessary."
        writeErr
        cOrEAndEnterToContinue
      fi
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END removeRmt0

# ---------------------------------------------------------------------------
# Copies the files which were tar'd up to their original locations
# ---------------------------------------------------------------------------

function restoreFilesToOriginalLocations {

  subprocessName='restoreFilesToOriginalLocations'
  if [ "$skipRestoreFilesToOriginalLocations" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    # -----------------------------------------------------------------------
    # Copy the TSM client configuration files back

    print "INFO: ($scriptName) - Copying TSM client config files to"
    print "                      $pathToTsmClientFiles:"
    print
    for eachFile in $tsmClientFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$restoreDir/$eachFile" ]
      then
        cp $restoreDir/$eachFile $pathToTsmClientFiles 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          tsmClientFilesFound="$tsmClientFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $restoreDir/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the TSM server configuration files back

    print "INFO: ($scriptName) - Copying TSM server config files to"
    print "                      $pathToTsmServerFiles:"
    print
    for eachFile in $tsmServerFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$restoreDir/$eachFile" ]
      then
        cp $restoreDir/$eachFile $pathToTsmServerFiles 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          tsmServerFilesFound="$tsmServerFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $restoreDir/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the first set of /etc configuration files back

    print "INFO: ($scriptName) - Copying etc config files to $pathToEtcFiles:"
    print
    for eachFile in $etcFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$restoreDir/$eachFile" ]
      then
        cp $restoreDir/$eachFile $pathToEtcFiles 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          etcFilesFound="$etcFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $restoreDir/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy the sysInfo configuration files back

    print "INFO: ($scriptName) - Copying sysInfo config files to"
    print "                      $pathToSysInfoFiles:"
    print
    for eachFile in $sysInfoFiles
    do
      print "                      $eachFile -- \c"
      if [ -f "$restoreDir/$eachFile" ]
      then
        cp $restoreDir/$eachFile $pathToSysInfoFiles 2>/dev/null
        cpRC=$?
        if [ "$cpRC" = '0' ]
        then
          print "successful"
          sysInfoFilesFound="$sysInfoFilesFound $eachFile "
        else
          print "!! FAILED !!"
          listOfFailedFiles="$listOfFailedFiles $restoreDir/$eachFile "
        fi
      else
        print "** MISSING **"
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy miscellaneous TSM server info files back

    stateOfMiscTsmServerFiles=`cat $restoreDir/$criticalSysInfo | grep TSMMISCFILESSAVED | awk '{print $2}'`
    if [ "$stateOfMiscTsmServerFiles" = '' ]
    then
      print "INFO: ($scriptName) - No TSM server accounting, fileexit, or userexit files"
      print "                      to restore."
      print
    elif [ "$stateOfMiscTsmServerFiles" = 'Manual' ]
    then
      print "INFO: ($scriptName) - You have opted to copy these files manually in Phase 1."
      print "                      Please open another window to restore them now before"
      print "                      continuing."
      print
      cAndEnterToContinue
    else
      print "INFO: ($scriptName) - Copying misc TSM files to original locations:"
      print
      listOfMiscTsmServerFiles=`cat $restoreDir/$criticalSysInfo | grep TSMMISCFILESSAVED | cut -f2- -d' '`
      for eachFile in $listOfMiscTsmServerFiles
      do
        print "                      $eachFile -- \c"
        if [ -f "$restoreDir/$eachFile" ]
        then
          cp $restoreDir/$eachFile $pathToSysInfoFiles 2>/dev/null
          cpRC=$?
          if [ "$cpRC" = '0' ]
          then
            print "successful"
            listOfMiscTsmServerFilesFound="$listOfMiscTsmServerFiles $eachFile "
          else
            print "!! FAILED !!"
            listOfFailedFiles="$listOfFailedFiles $restoreDir/$eachFile "
          fi
        else
          print "** MISSING **"
        fi
      done
      print
    fi

    # -----------------------------------------------------------------------
    # Copy TSM server DRM files back

    restoreDrmFiles

    # -----------------------------------------------------------------------
    # Copy SSA disk information files back

    print "INFO: ($scriptName) - Copying SSA disk information files to"
    print "                      $pathToSsaDiskFiles:"
    print
    rm $pathToSsaDiskFiles/* 2>/dev/null
    ssaDiskFiles=`ls $restoreDir/$relativePathToSsaDiskFiles`
    for eachFile in $ssaDiskFiles
    do
      print "                      $eachFile -- \c"
      cp $restoreDir/$relativePathToSsaDiskFiles/$eachFile $pathToSsaDiskFiles 2>/dev/null
      cpRC=$?
      if [ "$cpRC" = '0' ]
      then
        print "successful"
        ssaDiskFilesFound="$ssaDiskFilesFound $relativePathToSsaDiskFiles/$eachFile "
      else
        print "!! FAILED !!"
        listOfFailedFiles="$listOfFailedFiles $restoreDir/$relativePathToSsaDiskFiles/$eac
hFile "
      fi
    done
    print

    # -----------------------------------------------------------------------
    # Copy lsSSlevel (WebShlLvl.*) files back

    print "INFO: ($scriptName) - Copying lsSSlevel info files to $pathToSsLevelFiles:"
    print
    restoredSsLevelFiles=`ls $restoreDir/$relativePathToSsLevelFiles`
    for eachFile in $restoredSsLevelFiles
    do
      print "                      $eachFile -- \c"
      cp $restoreDir/$relativePathToSsLevelFiles/$eachFile $pathToSsLevelFiles 2>/dev/null
      cpRC=$?
      if [ "$cpRC" = '0' ]
      then
        print "successful"
        ssLevelFilesFound="$ssLevelFilesFound $relativePathToSsLevelFiles/$eachFile "
      else
        print "!! FAILED !!"
        listOfFailedFiles="$listOfFailedFiles $restoreDir/$relativePathToSsLevelFiles/$eac
hFile "
      fi
    done
    print

    subprocessName='restoreFilesToOriginalLocations'
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END restoreFilesToOriginalLocations

# ---------------------------------------------------------------------------
# Upgrades the TSM database
# ---------------------------------------------------------------------------

function upgradeDb {

  subprocessName='upgradeDb'
  if [ "$skipUpgradeDb" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Upgrading the TSM server database ..."
    print

    theCommand="cd $pathToTsmServerFiles"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand 2>/dev/null

    if [ -L core ]
    then
      print "INFO: ($scriptName) - core link already established"
    else
      if [[ -f $adsmlogDir/core ]]
      then
        timeStamp=`date +%m%d%Y_%H%M%S`
        print "INFO: ($scriptName) - Existing core file found in $adsmlogDir.  Renaming to"
        print "                      core.$timeStamp."
        print

        theCommand="mv $adsmlogDir/core $adsmlogDir/core.$timeStamp"
        print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
        print
        eval $theCommand
      fi

      if [[ -f $core ]]
      then
        timeStamp=`date +%m%d%Y_%H%M%S`
        print "INFO: ($scriptName) - Existing core file found in server directory.  Moving"
        print "                      to $adsmlogDir/core.$timeStamp."
        print

        theCommand="mv core $adsmlogDir/core.$timeStamp"
        print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
        print
        eval $theCommand
      fi

      print "INFO: ($scriptName) - Redirecting core files to /adsmlog ..."
      print

      theCommand="ln -s $adsmlogDir/core core"
      print "Issuing command: \"$theCommand\""
      print
      eval $theCommand
    fi

    theCommand="nohup dsmserv upgradedb &"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand > /dev/null

    sleepWithSpinner 180

    export adsmID=`cat $restoreDir/$criticalSysInfo | grep TSMADMINID | awk '{print $2}'`
    export adminUserID=$adsmID
    export adsmPSWD=`cat $restoreDir/$criticalSysInfo | grep TSMADMINPASSWORD | awk '{print $2}'`
    export adminPassword=$adsmPSWD
    export adsmServer="ss-m"
    export serverStanza=$adsmServer
    export tsmInvoc="dsmadmc -i -id=$adsmID -pa=$adsmPSWD -server=$adsmServer"

    verifyServerCommAutomatically

    stopPrimaryServer

    subprocessName='upgradeDb'
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END upgradeDb

# ---------------------------------------------------------------------------
# Restore network information
# ---------------------------------------------------------------------------

function restoreNetworkInformation {

  subprocessName='restoreNetworkInformation'
  if [ "$skipRestoreNetworkInformation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - System network information will now be restored.  It is"
    print "                      HIGHLY recommended an additional AIX terminal window be"
    print "                      opened for use during this process.  If there are any"
    print "                      problems restoring IP information you will NOT be able to"
    print "                      open up additional windows."
    print
    print "                      Do not close this additional window as it will be used"
    print "                      again."
    print
    cAndEnterToContinue

    export typeOfConversion='operatingSystem'
    theCommand="restoreIPenvironment $restoreDir/$captureIPenvironmentFile"
    print "                      Issuing command: \"$theCommand\""
    eval $theCommand
    theCommandRC=$?
    if [ "$theCommandRC" = '0' ]
    then
      print "INFO: ($scriptName) - Successfully restored system IP information."
      print
    else
      writeErr "NSM-95149-E: ERROR($scriptName) - Unable to restore system IP information!  Please take"
      writeErr "                                  corrective action."
      writeErr
      eAndEnterToExit
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END restoreNetworkInformation

# ---------------------------------------------------------------------------
# Issues "reg lic" commands against the TSM server
# $1 = 'alpha / numeric' ** $2 = license file name ** $3 = license value
# ---------------------------------------------------------------------------

function regLic {

  if [ "$1" = 'alpha' ]
  then
    $tsmInvoc reg lic file=$2 | grep AN
    regLicRC=$?
    if [ "$regLicRC" = '0' ]
    then
      print
    else
      writeErr
      writeErr "NSM-95150-E: ERROR($scriptName) - Unable to register TSM license!  Please take corrective action."
      writeErr
      eAndEnterToExit
    fi
  elif [ "$1" = 'numeric' ]
  then
    howManyTens=0
    howManyFives=0
    howManyOnes=0

    if [ "$3" -lt '10' ]
    then
      howManyOnes=$3
    elif [ "$3" -eq '10' ]
    then
      howManyTens=1
    elif [ "$3" -gt '10' ]
    then
      howManyTens=$(($3 / 10))
      subTotal=$(($howManyTens * 10))
      howManyLeft=$(($3 - $subTotal))

      if [ "$howManyLeft" -lt '5' ]
      then
        howManyOnes=$howManyLeft
      elif [ "$howManyLeft" -eq '5' ]
      then
        howManyFives=1
      elif [ "$howManyLeft" -gt '5' ]
      then
        howManyFives=$(($howManyLeft / 5))
        subTotal=$(($howManyFives * 5))
        howManyOnes=$(($howManyLeft - $subTotal))
      fi
    fi

    if [ "$howManyTens" = '0' ]
    then
      :  # Proceed normally
    else
      $tsmInvoc reg lic file=10$2 n=$howManyTens | grep AN
      regLicRC=$?
      if [ "$regLicRC" = '0' ]
      then
        print
      else
        writeErr
        writeErr "NSM-95151-E: ERROR($scriptName) - Unable to register TSM license!  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    fi

    if [ "$howManyFives" = '0' ]
    then
      :  # Proceed normally
    else
      $tsmInvoc reg lic file=5$2 n=$howManyFives | grep AN
      regLicRC=$?
      if [ "$regLicRC" = '0' ]
      then
        print
      else
        writeErr
        writeErr "NSM-95152-E: ERROR($scriptName) - Unable to register TSM license!  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    fi

    if [ "$howManyOnes" = '0' ]
    then
      :  # Proceed normally
    else
      $tsmInvoc reg lic file=1$2 n=$howManyOnes | grep AN
      regLicRC=$?
      if [ "$regLicRC" = '0' ]
      then
        print
      else
        writeErr
        writeErr "NSM-95153-E: ERROR($scriptName) - Unable to register TSM license!  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    fi
  fi

}  # END regLic

# ---------------------------------------------------------------------------
# Registers the TSM licenses collected in Phase 1 against the v4r2 server
# ---------------------------------------------------------------------------

function registerTsmLicenses {

  subprocessName='registerTsmLicenses'
  if [ "$skipRegisterTsmLicenses" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    licenseBundles=`cat $restoreDir/$criticalSysInfo | grep TSM41LICENSES | cut -f2- -d' '`

    for eachBundle in $licenseBundles
    do
      licenseName=`print $eachBundle | cut -f1 -d','`
      licenseValue=`print $eachBundle | cut -f2 -d','`

      if [ "$licenseName" = 'MGSYSLAN_LIC' ]
      then
        mgsyslan_lic="mgsyslan.lic"
        mgsyslan_lic_value=$licenseValue
        if [ "$mgsyslan_lic_value" = '5000' ] || [ "$mgsyslan_lic_value" = '5001' ]
        then
          :  # Proceed normally
        else
          regLic numeric $mgsyslan_lic $(($mgsyslan_lic_value - 5001))
        fi
      elif [ "$licenseName" = 'SPACEMAN_LIC' ]
      then
        spaceman_lic="spacemgr.lic"
        spaceman_lic_value=$licenseValue
        if [ "$spaceman_lic_value" != '0' ]
        then
          regLic numeric $spaceman_lic $spaceman_lic_value
        fi
      elif [ "$licenseName" = 'DRM_LIC' ]
      then
        drm_lic="drm.lic"
        drm_lic_value=$licenseValue
        if [ "$drm_lic_value" = 'Yes' ]
        then
          regLic alpha $drm_lic
        fi
      elif [ "$licenseName" = 'MGSYSSAN_LIC' ]
      then
        mgsyssan_lic="mgsyssan.lic"
        mgsyssan_lic_value=$licenseValue
        if [ "$mgsyssan_lic_value" != '0' ]
        then
          regLic numeric $mgsyssan_lic $mgsyssan_lic_value
        fi
      elif [ "$licenseName" = 'ORACLE_LIC' ]
      then
        oracle_lic="oracle.lic"
        oracle_lic_value=$licenseValue
        if [ "$oracle_lic_value" != '0' ]
        then
          regLic numeric $oracle_lic $oracle_lic_value
        fi
      elif [ "$licenseName" = 'MSSQL_LIC' ]
      then
        mssql_lic="mssql.lic"
        mssql_lic_value=$licenseValue
        if [ "$mssql_lic_value" != '0' ]
        then
          regLic numeric $mssql_lic $mssql_lic_value
        fi
      elif [ "$licenseName" = 'MSEXCH_LIC' ]
      then
        msexch_lic="msexch.lic"
        msexch_lic_value=$licenseValue
        if [ "$msexch_lic_value" != '0' ]
        then
          regLic numeric $msexch_lic $msexch_lic_value
        fi
      elif [ "$licenseName" = 'LNOTES_LIC' ]
      then
        lnotes_lic="lnotes.lic"
        lnotes_lic_value=$licenseValue
        if [ "$lnotes_lic_value" != '0' ]
        then
          regLic numeric $lnotes_lic $lnotes_lic_value
        fi
      elif [ "$licenseName" = 'DOMINO_LIC' ]
      then
        domino_lic="domino.lic"
        domino_lic_value=$licenseValue
        if [ "$domino_lic_value" != '0' ]
        then
          regLic numeric $domino_lic $domino_lic_value
        fi
      elif [ "$licenseName" = 'INFORMIX_LIC' ]
      then
        informix_lic="informix.lic"
        informix_lic_value=$licenseValue
        if [ "$informix_lic_value" != '0' ]
        then
          regLic numeric $informix_lic $informix_lic_value
        fi
      elif [ "$licenseName" = 'SAPR3_LIC' ]
      then
        sapr3_lic="r3.lic"
        sapr3_lic_value=$licenseValue
        if [ "$sapr3_lic_value" != '0' ]
        then
          regLic numeric $sapr3_lic $sapr3_lic_value
        fi
      elif [ "$licenseName" = 'ESS_LIC' ]
      then
        ess_lic="ess.lic"
        ess_lic_value=$licenseValue
        if [ "$ess_lic_value" != '0' ]
        then
          regLic numeric $ess_lic $ess_lic_value
        fi
      elif [ "$licenseName" = 'ESSR3_LIC' ]
      then
        essr3_lic="essr3.lic"
        essr3_lic_value=$licenseValue
        if [ "$essr3_lic_value" != '0' ]
        then
          regLic numeric $essr3_lic $essr3_lic_value
        fi
      elif [ "$licenseName" = 'EMCSYMM_LIC' ]
      then
        emcsymm_lic="emcsymm.lic"
        emcsymm_lic_value=$licenseValue
        if [ "$emcsymm_lic_value" != '0' ]
        then
          regLic numeric $emcsymm_lic $emcsymm_lic_value
        fi
      elif [ "$licenseName" = 'EMCSYMR3_LIC' ]
      then
        emcsymr3_lic="emcsymr3.lic"
        emcsymr3_lic_value=$licenseValue
        if [ "$emcsymr3_lic_value" != '0' ]
        then
          regLic numeric $emcsymr3_lic $emcsymr3_lic_value
        fi
      elif [ "$licenseName" = 'LIBSHARE_LIC' ]
      then
        libshare_lic="libshare.lic"
        libshare_lic_value=$licenseValue
        if [ "$libshare_lic_value" = 'Yes' ]
        then
          regLic alpha $libshare_lic
        fi
      elif [ "$licenseName" = 'LIBRARY_LIC' ]
      then
        library_lic="library.lic"
        library_lic_value=$licenseValue
        if [ "$library_lic_value" != '2' ]
        then
          regLic numeric $library_lic $(($library_lic_value - 2))
        fi
      fi
    done

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END registerTsmLicenses

# ---------------------------------------------------------------------------
# Perform tape reconciliation and allow user to define tape hardware to TSM
# ---------------------------------------------------------------------------

function performTapeReconciliation {

  subprocessName='performTapeReconciliation'
  if [ "$skipPerformTapeReconciliation" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    touch /tmp/$tapeRecMarkerFile 2>/dev/null

    if [ "$theMachineType" = 'H70' ] && [ "$modelNumber" = 'C01' ]
    then
      print "INFO: ($scriptName) - Collecting tape information ..."
      print

      theCommand="getTapeInfo -q -e $restoreDir $getTapeInfoAfterFile"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
      theCommandRC=$?
      if [ "$theCommandRC" = '0' ]
      then
        print
      else
        writeErr "NSM-95154-E: ERROR($scriptName) - Unable to collect current tape information!"
        writeErr "                                  Please call IBM support."
        writeErr
        eAndEnterToExit
      fi

      print "INFO: ($scriptName) - Performing tape reconciliation ..."
      print

      theCommand="tapeRec -n $restoreDir/$getTapeInfoBeforeFile $restoreDir/$getTapeInfoAfterFile"
      print "INFO: ($scriptName) - Issuing command: \"$theCommand\""
      print
      eval $theCommand
      if [ "$theCommandRC" = '0' ]
      then
        print
        function displayHeader {
          print
          print "+-----------------------------------------------------------------------------+" 
          print "|                                                                             |"
          print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
          print "|                                                                             |"
          print "|                                                                             |"
          print "| If tape reconciliation completed successfully, you may continue without     |"
          print "| further modification to TSM.                                                |"
          print "|                                                                             |"
          print "| If tape reconciliation DID NOT complete successfully, you should exit the   |"
          print "| script now and check device definitions manually in TSM to ensure they      |"
          print "| match the settings in AIX and are properly defined.                         |"
          print "|                                                                             |"
          print "|                                                                             |"
          print "| NOTE: The script will restart with automated tape reconciliation and you    |"
          print "|       will be presented with this prompt again.                             |"
          print "|                                                                             |"
          print "|                                                                             |"
          print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
          print "|                                                                             |"
          print "+-----------------------------------------------------------------------------+" 
          print
          print
        }  # END displayHeader

        function displayMenu {
          print "[1] Yes, devices in AIX match the definitions in TSM.  Continue."
          print
          print "[2] No, I want to exit this script."
          print
        }  # END displayMenu

        function userSelect {
          displayHeader
          displayMenu
          response=0
          until [ "$response" = '1' ] || [ "$response" = '2' ]
          do
            read response?"Please make your selection [1,2]: "
            if [ "$response" = '1' ] || [ "$response" = '2' ]
            then
              : # Pass through
            else
              displayHeader
              print "-> $response <- is an invalid response!  Please try again."
              print
              print
              displayMenu
            fi
          done

          case $response in
            (1)
              print
              print "INFO: ($scriptName) - User selected Option [1] -- Continuing script."
              print
              ;;
            (2)
              print
              print "INFO: ($scriptName) - User selected Option [2] -- Exiting script."
              print
              programRC=1
              exit $programRC
              ;;
          esac
        }  # END userSelect

        userSelect
      else
        writeErr "NSM-95155-E: ERROR($scriptName) - Unable to perform automatic tape reconciliation!"
        writeErr "                                  Please call IBM support."
        writeErr
        eAndEnterToExit
      fi
    elif [ "$theMachineType" = 'H50' ] && [ "$modelNumber" = 'C00' ]
    then
      function displayHeader {
        print
        print "+-----------------------------------------------------------------------------+" 
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "| This is a C00 system, so tape information was not collected in Phase 1.     |"
        print "| You should exit the script now and perform tape reconciliation in TSM to    |"
        print "| ensure devices match the settings in AIX and are defined properly.          |"
        print "|                                                                             |"
        print "| Once you have verified devices are defined properly in TSM, you may restart |" 
        print "| the script and continue from this point.                                    |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "| NOTE: C00 systems converted from C10, C20, or C30 models MUST pay           |"
        print "|       particular attention to this step.                                    |"
        print "|                                                                             |"
        print "|                                                                             |"
        print "|  *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT *** IMPORTANT ***  |"
        print "|                                                                             |"
        print "+-----------------------------------------------------------------------------+" 
        print
        print
      }  # END displayHeader

      function displayMenu {
        print "[1] Yes, devices in AIX match the definitions in TSM.  Continue."
        print
        print "[2] No, I want to exit this script."
        print
      }  # END displayMenu

      function userSelect {
        displayHeader
        displayMenu
        response=0
        until [ "$response" = '1' ] || [ "$response" = '2' ]
        do
          read response?"Please make your selection [1,2]: "
          if [ "$response" = '1' ] || [ "$response" = '2' ]
          then
            : # Pass through
          else
            displayHeader
            print "-> $response <- is an invalid response!  Please try again."
            print
            print
            displayMenu
          fi
        done

        case $response in
          (1)
            print
            print "INFO: ($scriptName) - User selected Option [1] -- Continuing script."
            print
            ;;
          (2)
            print
            print "INFO: ($scriptName) - User selected Option [2] -- Exiting script."
            print
            programRC=1
            exit $programRC
            ;;
        esac
      }  # END userSelect

      userSelect
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END performTapeReconciliation

# ---------------------------------------------------------------------------
# Enable TSM server schedules
# ---------------------------------------------------------------------------

function enableSchedules {

  subprocessName='enableSchedules'
  if [ "$skipEnableSchedules" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Enabling TSM server schedules ..."
    print

    realFile="$pathToTsmServerFiles/dsmserv.opt"
    workFile="$pathToTsmServerFiles/dsmserv.opt.tmp"
    somethingHappened='no'

    theCommand="cp $realFile $workFile"
    eval $theCommand 2>/dev/null
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingHappened='yes'
    fi

    theCommand="sed -e "/DISABLESCheds/d" $realFile > $workFile"
    eval $theCommand 2>/dev/null
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingHappened='yes'
    fi

    theCommand="cp $workFile $realFile"
    eval $theCommand 2>/dev/null
    evalRC=$?
    if [ "$evalRC" != '0' ]
    then
      somethingHappened='yes'
    fi

    if [ "$somethingHappened" = 'no' ]
    then
      print "INFO: ($scriptName) - TSM server schedules successfully enabled."
      print
    else
      writeErr "NSM-95156-W: WARNING($scriptName) - Could not enable TSM server schedules."
      writeErr "                                    Please take corrective action."
      writeErr
      cOrEAndEnterToContinue
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END enableSchedules

# ---------------------------------------------------------------------------
# Audit the TSM server licenses after the nodelock file has been copied back
# ---------------------------------------------------------------------------

function auditTsmLicenses {

  subprocessName='auditTsmLicenses'
  if [ "$skipAuditTsmLicenses" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Initiating TSM server license auditing process ..."
    print

    theCommand="$tsmInvoc aud lic"
    eval $theCommand > /dev/null

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END auditTsmLicenses

# ---------------------------------------------------------------------------
# Let all TSM server processes finish before continuing
# ---------------------------------------------------------------------------

function allowTsmProcessesToFinish {

  subprocessName='allowTsmProcessesToFinish'
  if [ "$skipAllowTsmProcessesToFinish" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Allowing TSM server processes to finish ..."
    print

    $tsmInvoc q proc > /dev/null
    procRC=$?
    while [ "$procRC" != '11' ]
    do
      $tsmInvoc q proc > /dev/null
      procRC=$?
      sleepWithSpinner 60

      isitup primary > /dev/null
      isitupRC=$?
      if [ "$isitupRC" = '0' ]
      then
        :  # Proceed normally
      else
        writeErr ()
        {
          print "$*" >&2
        }
        writeErr "NSM-95157-E: ERROR($scriptName) - The TSM server is no longer running!"
        writeErr "                                  Please take corrective action."
        writeErr
        eAndEnterToExit
      fi
    done

    print "INFO: ($scriptName) - No TSM server processes found."
    print

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END allowTsmProcessesToFinish

# ---------------------------------------------------------------------------
# Stamp the EC level (create the lsSSlevel WebShlLvl* file in /var/adm)
# ---------------------------------------------------------------------------

function applyEcStamp {

  subprocessName='applyEcStamp'
  if [ "$skipApplyEcStamp" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    mv $pathToSsLevelFiles/$ssLevelFiles /tmp/$ssLevelFiles.prebuild 2>/dev/null

    installDate=`date +"%m%d%Y"`
    installTime=`date +"%H%M"`
    ecFilename="WebShlLvl.$installDate$installTime"

    printf "$thisEcLevel: Filesets installed for EC level \n" > $pathToSsLevelFiles/$ecFilename
    printf "Date installed is $installDate\n" >> $pathToSsLevelFiles/$ecFilename
    lslpp -L all >> $pathToSsLevelFiles/$ecFilename

    isItStamped=`lsSSlevel | grep $thisEcLevel | awk '{print $1}'`
    if [ "$isItStamped" = '' ]
    then
      writeErr ()
      {
        print "$*" >&2
      }
      writeErr "NSM-95158-E: ERROR($scriptName) - Unable to generate EC stamp for this machine!"
      writeErr "                                  Please contact IBM support for further assitance."
      writeErr
      eAndEnterToExit
    else
      print "INFO: ($scriptName) - EC level successfully stamped."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END applyEcStamp

# ---------------------------------------------------------------------------
# Call record.ssa to record all the SSA disk information
# ---------------------------------------------------------------------------

function callRecordSsa {

  subprocessName='callRecordSsa'
  if [ "$skipCallRecordSsa" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeEntering

    theCommand="record.ssa all"
    print "                      Issuing command: \"$theCommand\""
    print
    eval $theCommand 2>/dev/null
    theCommandRC=$?

    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeScript $subprocessTypeSkipping
  fi

}  # END callRecordSsa

# ---------------------------------------------------------------------------
# Add a nodes.dat file because apparently AIX does not
# ---------------------------------------------------------------------------

function addNodesDat {

  subprocessName='addNodesDat'
  if [ "$skipAddNodesDat" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    if [ -f "$pathToNodesDatFile/$nodesDatFile" ]
    then
      print "INFO: ($scriptName) - A $nodesDatFile file already exists in"
      print "                      $pathToNodesDatFile."
      print
    else
      touch $pathToNodesDatFile/$nodesDatFile
      chmod 777 $pathToNodesDatFile/$nodesDatFile

      print "INFO: ($scriptName) - Added $nodesDatFile file to"
      print "                      $pathToNodesDatFile."
      print
    fi

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END addNodesDat

# ---------------------------------------------------------------------------
# Closes out Phase 2 and performs a shutdown -Fr if desired
# ---------------------------------------------------------------------------

function endPhase2 {

  subprocessName='endPhase2'
  if [ "$skipEndPhase2" = 'no' ]
  then
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeEntering

    print "INFO: ($scriptName) - Adding automatic start of the TSM server ..."
    print

    autoTsmStatus=`lsitab -a | grep autosrvr`
    if [ "$autoTsmStatus" = '' ]
    then
      mkitab autosrvr:2:once:"/usr/nsm.3466/script/startadsm all" 2>/dev/null
      addautoRC=$?
      if [ "$addautoRC" = '0' ]
      then
        print "INFO: ($scriptName) - Successfully added entry to initialization table."
        print
      else
        print "NSM-95159-W: WARNING($scriptName) - Could not add entry to initialization table."
        print
        cOrEAndEnterToContinue
      fi
    else
      print "INFO: ($scriptName) - An entry was already found in the initialization table."
      print
    fi

    # -----------------------------------------------------------------------
    # Get the original AIX hostname to rename the /tmp/storsmart/XXX dir
    # since it's used as a check if Phase 2 was previously installed

    realHostName=`cat $restoreDir/$criticalSysInfo | grep "Critical System Information for" | awk '{print $6}'`

    function displayHeader {
      print
      print "+-----------------------------------------------------------------------------+"
      print "|                                                                             |"
      print "| Phase 2 has completed successfully.  Please take note of the following      |"
      print "| directories:                                                                |"
      print "|                                                                             |"
      print "|                                                                             |"
      print "| *** Phase 1 Console Log(s) ***                                              |"
      print "|                                                                             |"
      print "|   -> $prepareDir/$scriptName.phase1/                                   |"
      print "|                                                                             |"
      print "|                                                                             |"
      print "| *** Phase 1 Save Files ***                                                  |"
      print "|                                                                             |"
      print "|   -> $prepareDir/                                                      |"
      print "|   -> $restoreDir/  (restored from diskette)                     |"
      print "|                                                                             |"
      print "|                                                                             |"
      print "| *** Phase 2 Console Log(s) ***                                              |"
      print "|                                                                             |"
      print "|   -> $outputDir/                                                      |"
      print "|                                                                             |"
      print "|                                                                             |"
      print "| It is advisable to save the files contained in these directories to another |"
      print "| location if diagnostic work is required at a later date.                    |"
      print "|                                                                             |"
      print "| Please remove any remaining media (4mm tape, diskette) from the system and  |"
      print "| restart to complete the upgrade.                                            |"
      print "|                                                                             |"
      print "+-----------------------------------------------------------------------------+"
      print
      print
    }  # END displayHeader

    function displayMenu {
      print "[1] Restart the system with the \"shutdown -Fr\" command."
      print
      print "[2] No, I want to exit this script now and restart later."
      print
    }  # END displayMenu

    displayHeader
    response=0
    until [ "$response" = '1' ] || [ "$response" = '2' ]
    do
      displayMenu
      read response?"Please make your selection [1,2]: "
      if [ "$response" = '1' ] || [ "$response" = '2' ]
      then
        : # Pass through
      else
        displayHeader
        print "-> $response <- is an invalid response!  Please try again."
      fi
      print
      print
    done

    case $response in
      (1)
        print "INFO: ($scriptName) - User selected Option [1] -- Restart system now."
        print
        mv /tmp/storsmart/localhost /tmp/storsmart/$realHostName 2>/dev/null
        shutdown -Fr
        ;;
      (2)
        print "INFO: ($scriptName) - User selected Option [2] -- Restart system later."
        print
        mv /tmp/storsmart/localhost /tmp/storsmart/$realHostName 2>/dev/null
        programRC=0
        return $programRC
        ;;
    esac

    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeExiting
  else
    subprocessStatus $subprocessName $subprocessTypeFunction $subprocessTypeSkipping
  fi

}  # END endPhase2

# ---------------------------------------------------------------------------
# Pre-analysis Phase Functions
# ---------------------------------------------------------------------------

function preAnalysisPhase {

  print "INFO: ($scriptName) > Entering Pre-Analysis Phase."
  print

  checkForExistingEc
  determineProcessorInfo
  determineModelNumber

  if [ "$currentPhase" = '1' ]
  then
    print "INFO: ($scriptName) - System hostname is $hostName."
    print

    displayKinServerWarning
    verifyNsmPackageLevel
    verifyNsmMainDir
    verifyAdsmlogDir
    createCriticalSysInfoHeader
    startPrimaryServer
    verifyServerCommunications
    assignTsmAdminVariables
    verifyApprovedTsmLevel
    checkForFullDatabaseBackup
    checkRootVgFilesystems
    callVerify3466Scripts
  fi

  print "INFO: ($scriptName) < Exiting Pre-Analysis Phase."
  print

}  # END preAnalysisPhase

# ---------------------------------------------------------------------------
# Phase 1 Functions
# ---------------------------------------------------------------------------

function phase1 {

  print "INFO: ($scriptName) > Entering Phase 1."
  print

  gatherSysInfo
  saveTimeZone
  collectTsmLicenseInformation

  if [[ -f $adsmlogDir/$storagePoolMigration ]]
  then
    enableMigration
  fi

  disableMigration
  disableReclamation
  disableSchedules
  disableSessions
  collectTapeInformation
  collectNetworkInformation
  checkMiscTsmServerFileSizes
  saveFilesToFloppy
  stopPrimaryServer
  prebackupExportit
  endPhase1

  print "INFO: ($scriptName) < Exiting Phase 1."
  print

}  # END phase1

# ---------------------------------------------------------------------------
# Phase 2 Functions
# ---------------------------------------------------------------------------

function phase2 {

  print "INFO: ($scriptName) > Entering Phase 2."
  print

  function phase2part1 {
    callFixSsa
    restoreFilesFromFloppy
    checkSystemDateTz
    postbackupImportit
    checkSsa
    removeExtraFiles
    checkRootVgFilesystems
    removeRmt0
    restoreFilesToOriginalLocations
    updateModel
    checkRootVgFilesystems
    assignTsmAdminVariables
    upgradeDb
    restoreNetworkInformation
    verifyServerCommAutomatically
    registerTsmLicenses
    disableSessions
  }  # END phase2part1

  function phase2part2 {
    assignTsmAdminVariables
    performTapeReconciliation
    enableSchedules
    auditTsmLicenses
    allowTsmProcessesToFinish
    enableMigration
    enableReclamation
    applyEcStamp
    callRecordSsa
    addNodesDat
    endPhase2
  }  # END phase2part2

  if [ ! -f "/tmp/$tapeRecMarkerFile" ]
  then
    phase2part1
    phase2part2
  else
    print "INFO: ($scriptName) - Tape reconciliation marker file was found in /tmp."
    print "                      Resuming Phase 2 ..."
    print
    phase2part2
  fi

  print "INFO: ($scriptName) < Exiting Phase 2."
  print

}  # END phase2

# ---------------------------------------------------------------------------
# Main menu
# ---------------------------------------------------------------------------

function mainMenu {

  function displayHeader {
    echo
    echo "+-----------------------------------------------------------------------------+"
    echo "|                                                                             |"
    echo "| Make a selection to run the required phase:                                 |"
    echo "|                                                                             |"
    echo "|   [1]  Phase 1:                                                             |"
    echo "|                                                                             |"
    echo "|     -- Save critical files                                                  |"
    echo "|     -- Quiesce TSM server activities                                        |"
    echo "|     -- Save tape information                                                |"
    echo "|        (C01's with IBM 3494, 3575, 3583 or 3584 only)                       |"
    echo "|     -- Save network information                                             |"
    echo "|                                                                             |"
    echo "|   Phase 1 is run prior to modifying the 3466 system.  Problems encountered  |"
    echo "|   MUST be corrected prior to continuing.                                    |"
    echo "|                                                                             |"
    echo "|   [2]  Phase 2:                                                             |"
    echo "|                                                                             |"
    echo "|     -- Restore critical files and configuration data                        |"
    echo "|                                                                             |"
    echo "|   [3]  Exit                                                                 |"
    echo "|                                                                             |"
    echo "| NOTE: Both Phase 1 and 2 may be re-started in the event the script does not |"
    echo "|       complete in a previous instance.  However once a phase has completed  |"
    echo "|       successfully, it may NOT be re-started.                               |"
    echo "|                                                                             |"
    echo "+-----------------------------------------------------------------------------+"
    echo
    echo
  }  # END displayHeader

  function displayMenu {
    echo "[1] Phase 1"
    echo
    echo "[2] Phase 2"
    echo
    echo "[3] Exit"
    echo
  }  # END displayMenu

  displayHeader
  response=0
  until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
  do
    displayMenu
    read response?"Please make your selection [1,2,3]: "
    if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
    then
      : # Pass through
    else
      displayHeader
      echo "-> $response <- is an invalid response!  Please try again."
    fi
    echo
    echo
  done

  case $response in
    (1)
      echo "INFO: ($scriptName) - User selected Option [1] -- Phase 1."
      echo

      if [ "$currentPhase" = '2' ]
      then
        writeErr "NSM-95150-E: ERROR($scriptName) - Phase 1 has already been run!  Please restart this script and"
        writeErr "                                  select Phase 2."
        writeErr
        eAndEnterToExit
      fi

      whichPhase='phase1'
      phase1Complete='no'
      phase1
      phase1Complete='yes'
      programRC=0
      return $programRC
      ;;
    (2)
      echo "INFO: ($scriptName) - User selected Option [2] -- Phase 2."
      echo
      whichPhase='phase2'
      phase2Complete='no'
      phase2
      phase2Complete='yes'
      programRC=0
      return $programRC
      ;;
    (3)
      echo "INFO: ($scriptName) - User selected Option [3] -- Exit."
      echo
      programRC=0
      return $programRC
      ;;
  esac

}  # END mainMenu

# -----------------------------------------------------------------------------
# Mainline
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Process Special Options
#
# 1) Special options must start with a '-'.
# 2) The special option must be adjacent to the '-' and can be any number of
#    characters.
# 3) Any number of values can be specified for a special option.  If values
#    are processed, they must be captured and then shifted left.
# 4) All special options must be specified prior to positional parameters.
# -----------------------------------------------------------------------------

firstCharParm1=`echo $1 | cut -b1`
while [ "$firstCharParm1" = '-' ]
do
  specialOption=`echo $1 | cut -b2-`
  case $specialOption in
    ('n')
      noLoggingOption='yes'
      : ;;
    ('h')
      helpOption='yes'
      : ;;
    (*)
      writeErr
      writeErr "NSM-95151-E: ERROR($scriptName) - Special option: \"$1\" not recognized."
      writeErr "             Valid special options are '-n' and '-h'."
      writeErr
      eAndEnterToExit
      : ;;
  esac
  shift 1
  firstCharParm1=`echo $1 | cut -b1`
done

# -----------------------------------------------------------------------------
# Write help text as appropriate
# -----------------------------------------------------------------------------

if [ "$helpOption" = 'yes' ]
then
  clear
  cat $0 | grep \#@ | grep -v 'grep #@' | more
  exit
fi

# -----------------------------------------------------------------------------
# Begin execution
# -----------------------------------------------------------------------------

mainProcedure ()
{

  enterTimeStamp=`date +'%D %T'`
  print
  print "INFO: ($scriptName) - ENTERING $enterTimeStamp"
  print

  # ---------------------------------------------------------------------------
  # Tell the user where the console log will be kept if revfile is used

  if [ "$noLoggingOption" = 'no' ]
  then
    print "INFO: ($scriptName) - The console log can be found in:"
    print
    print "                      $theConsoleLog"
    print
  fi

  # ---------------------------------------------------------------------------
  # Execution

  expandScreen

  whichPhase='preanalysis'
  preanalysisPhaseComplete='no'
  preAnalysisPhase
  preanalysisPhaseComplete='yes'

  mainMenu

  # ---------------------------------------------------------------------------
  # Say goodbye

  exitTimeStamp=`date +'%D %T'`
  print "INFO: ($scriptName) - EXITING $exitTimeStamp (RC=$programRC)"
  print

  exit $programRC

}  # END mainProcedure

# -----------------------------------------------------------------------------
# Setup variables for console logging
# -----------------------------------------------------------------------------

timeStamp=`date +%m%d%Y_%H%M%S`
outputDir="/tmp/$scriptName"
mkdir -p $outputDir >/dev/null 2>&1
theConsoleLog="$outputDir/$scriptName.log.$timeStamp"

if [ "$noLoggingOption" = 'yes' ]
then
  # ---------------------------------------------------------------------------
  # The no logging function is used when called from another program, for
  # instance when called from replace.raid
  # ---------------------------------------------------------------------------

  mainProcedure $*
else
  # --------------------------------------------------------------------
  # Setup trap for CTRL-C
  # --------------------------------------------------------------------

  userInteruptString="
INFO: ($scriptName) - User entered CTRL-C to terminate program."
  trap 'echo "$userInteruptString" | tee -a "$theConsoleLog"' INT

  # --------------------------------------------------------------------
  # invoke the main procedure to do the work
  # --------------------------------------------------------------------

  mainProcedure $* 2>&1 | tee $theConsoleLog

  # --------------------------------------------------------------------
  # let user review the report
  # --------------------------------------------------------------------

  archiveDirectory='/tmp/savefiles'
  mkdir -p $archiveDirectory >/dev/null 2>&1

  revfile -c -a $archiveDirectory $theConsoleLog
fi
